@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -76,8 +76,10 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(pd()->jcp_.post_ops, ctx);
 
-    DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
-    DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper dst_d(pd()->dst_md());
@@ -95,13 +97,12 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     assert(jcp.is_relo);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
 
-    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
-    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
-    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
-
-    const float *oscales = scale_utils::precompute_scales(
-            ctx.get_scratchpad_grantor(), src_scales, wei_scales, pd()->OC(),
-            pd()->attr(), jit_scale_precompute_.get());
+    const void *src_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const void *wei_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const void *dst_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
 
     auto inp_p_buffer = ctx.get_scratchpad_grantor().template get<char>(
             key_conv_amx_inp_buffer); // fix the template
@@ -136,7 +137,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     const int zp_pbuff_size = jcp.zp_pbuff_size;
 
     // reorder weights from (g)Owhi16o to (g)OR16r16o4r, where r := whi
-    auto p = jit_conv_call_s();
+    auto p = jit_conv_args_t();
     p.src = weights;
     p.dst = wei_buffer;
     kernel_->copy_to_wbuffer()(&p);
@@ -165,7 +166,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
         const int oh_work = jcp.oh_pad;
         parallel_nd(
                 ngroups, oc_chunks, oh_work, [&](dim_t g, dim_t occ, dim_t oh) {
-                    auto p = jit_conv_call_s();
+                    auto p = jit_conv_args_t();
 
                     const int oh_ = oh >= zp_buff_b_pad_start
                             ? b_pad_start + oh - zp_buff_b_pad_start
@@ -190,7 +191,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                     p.oc_blocks = occ * jcp.nb_oc_blocking;
                     p.filt = weights
                             + wei_dt_size * (g * oc_chunks + occ) * wei_oc_step;
-                    p.src_zero_point = src_zero_point;
+                    p.src_zero_point = src_zero_points;
 
                     kernel_->zp_pbuff_kernel()(&p);
                 });
@@ -215,9 +216,20 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                 zp_flags[oc] = true;
         }
 
-        auto p = jit_conv_call_s();
+        auto p = jit_conv_args_t();
         amx_tile_configure(tcfg);
 
+        float *dst_scales_inv_ptr = nullptr;
+        if (jcp.with_dst_scales) {
+            const float *dst_scales_ptr
+                    = static_cast<const float *>(dst_scales);
+            dst_scales_inv_ptr
+                    = ctx.get_scratchpad_grantor().template get<float>(
+                              key_conv_dst_scales)
+                    + ithr;
+            dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+        }
+
         const int oh_work = jcp.oh_pad;
         const int sp_stride = mem_blk_off(dst_d, 0, 0, 0, 0, 1);
         const int dilate_h = jcp.dilate_h + 1;
@@ -248,8 +260,8 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                     : nullptr;
             p.zp_compensation
                     = jcp.src_zero_point ? zp_compensation + oc : nullptr;
-            p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
-            p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
+            p.src_zero_point = src_zero_points;
+            p.dst_zero_point = dst_zero_points;
 
             int oh_s = ohc * jcp.oh_blk_size;
             int oh_e = nstl::min(jcp.oh, oh_s + jcp.oh_blk_size);
@@ -301,7 +313,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                     p.oc_blocks = occ * jcp.nb_oc_blocking;
                     p.filt = weights
                             + wei_dt_size * (g * oc_chunks + occ) * wei_oc_step;
-                    p.src_zero_point = src_zero_point;
+                    p.src_zero_point = src_zero_points;
 
                     kernel_->zp_pbuff_kernel()(&p);
                 }
@@ -373,8 +385,12 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                 p.filt = wei
                         + wei_dt_size * (g * oc_chunks + occ) * wei_oc_shift;
                 p.bias = bias_w;
-                p.scales = &oscales[jcp.is_oc_scale * oc];
-                p.dst_scale = &dst_scales[0];
+                p.src_scales = src_scales;
+                p.wei_scales = jcp.with_wei_scales
+                        ? static_cast<const float *>(wei_scales)
+                                + jcp.is_oc_scale * oc
+                        : nullptr;
+                p.dst_scales = dst_scales_inv_ptr;
 
                 p.acc_s32 = wsp + ithr * jcp.wsp_buffer_size;
 
@@ -432,8 +448,10 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
     const memory_desc_wrapper bias_d(pd()->weights_md(1));
 
-    DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
-    DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
     const size_t bia_dt_size = pd()->with_bias()
             ? types::data_type_size(pd()->desc()->bias_desc.data_type)
@@ -450,13 +468,12 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
     const auto &jcp = pd()->jcp_;
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
 
-    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
-    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
-    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
-
-    const float *oscales = scale_utils::precompute_scales(
-            ctx.get_scratchpad_grantor(), src_scales, wei_scales, pd()->OC(),
-            pd()->attr(), jit_scale_precompute_.get());
+    const void *src_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const void *wei_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const void *dst_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
 
     // TODO: use block offset instead of hand-calculated one
     //size_t wei_oc_shift = wht_blk_off(weights_d, 0, 1);
@@ -520,7 +537,7 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
         const int oh_work = jcp.oh_pad;
         parallel_nd(ngroups, oc_chunks, od_work, oh_work,
                 [&](dim_t g, dim_t occ, dim_t od, dim_t oh) {
-                    auto p = jit_conv_call_s();
+                    auto p = jit_conv_args_t();
 
                     const int od_ = od >= zp_buff_back_pad_start
                             ? back_pad_start + od - zp_buff_back_pad_start
@@ -560,7 +577,7 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                     p.filt = weights
                             + wei_dt_size * (g * oc_chunks + occ)
                                     * wei_oc_shift;
-                    p.src_zero_point = src_zero_point;
+                    p.src_zero_point = src_zero_points;
 
                     kernel_->zp_pbuff_kernel()(&p);
                 });
@@ -585,9 +602,20 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                 zp_flags[oc] = true;
         }
 
-        auto p = jit_conv_call_s();
+        auto p = jit_conv_args_t();
         amx_tile_configure(tcfg);
 
+        float *dst_scales_inv_ptr = nullptr;
+        if (jcp.with_dst_scales) {
+            const float *dst_scales_ptr
+                    = static_cast<const float *>(dst_scales);
+            dst_scales_inv_ptr
+                    = ctx.get_scratchpad_grantor().template get<float>(
+                              key_conv_dst_scales)
+                    + ithr;
+            dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+        }
+
         const int oh_work = jcp.oh_pad;
         const int sp_stride = mem_blk_off(dst_d, 0, 0, 0, 0, 1);
         const int dilate_d = jcp.dilate_d + 1;
@@ -617,8 +645,8 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                     : nullptr;
             p.zp_compensation
                     = jcp.src_zero_point ? zp_compensation + oc : nullptr;
-            p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
-            p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
+            p.src_zero_point = src_zero_points;
+            p.dst_zero_point = dst_zero_points;
 
             const size_t inp_src_d_stride = mem_blk_off(src_d, 0, 0, 1, 0, 0);
             int oh_s = ohc * jcp.oh_blk_size;
@@ -673,7 +701,7 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                     p.filt = weights
                             + wei_dt_size * (g * oc_chunks + occ)
                                     * wei_oc_shift;
-                    p.src_zero_point = src_zero_point;
+                    p.src_zero_point = src_zero_points;
 
                     kernel_->zp_pbuff_kernel()(&p);
                 }
@@ -765,8 +793,12 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                                   + d_f_overflow * wei_d_shift)
                                 * wei_dt_size;
                 p.bias = bias_w;
-                p.scales = &oscales[jcp.is_oc_scale * oc];
-                p.dst_scale = &dst_scales[0];
+                p.src_scales = src_scales;
+                p.wei_scales = jcp.with_wei_scales
+                        ? static_cast<const float *>(wei_scales)
+                                + jcp.is_oc_scale * oc
+                        : nullptr;
+                p.dst_scales = dst_scales_inv_ptr;
 
                 p.acc_s32 = wsp + ithr * jcp.wsp_buffer_size;
                 if (req_zero_point_buffer
@@ -825,9 +857,11 @@ status_t jit_avx512_core_amx_convolution_bwd_data_t::execute_backward(
     DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
     DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
 
+    const int wei_scale_mask = pd()->attr()->scales_.get_mask(DNNL_ARG_WEIGHTS);
     const float *oscales = scale_utils::precompute_scales(
-            ctx.get_scratchpad_grantor(), src_scales, wei_scales, pd()->OC(),
-            pd()->attr(), jit_scale_precompute_.get());
+            ctx.get_scratchpad_grantor(), src_scales, wei_scales, pd()->IC(),
+            pd()->OC(), false, wei_scale_mask > 0, pd()->attr(),
+            jit_scale_precompute_.get());
 
     amx_utils::execute_backward_convolution_body(ctx, pd()->jcp_, kernel_,
             diff_dst, weights, nullptr /* no bias */, oscales, dst_scales,
@@ -860,8 +894,8 @@ status_t jit_avx512_core_amx_convolution_bwd_weights_t::init(engine_t *engine) {
     }
     if (j.transform_to_vnni) {
         CHECK(safe_ptr_assign(diff_wei_trans_kernel_,
-                new jit_diff_wei_trans_to_vnni_t(
-                        j.wei_dt, j.kd, j.kh, j.kw, j.ic_block, j.oc_block)));
+                new jit_diff_wei_trans_to_vnni_t(j.wei_dt, j.kd, j.kh, j.kw,
+                        j.ic_block, j.oc_block, j.nb_ic)));
         CHECK(diff_wei_trans_kernel_->create_kernel());
     }
     return status::success;
@@ -1100,7 +1134,7 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::compute_diff_weights_2d(
     nd_iterator_init(start, img, jcp.mb, oh_s, jcp.oh);
 
     while (start < end) {
-        auto p = jit_conv_call_s();
+        auto p = jit_conv_args_t();
         int work_rem = end - start;
         const int oh_e = oh_s + work_rem > jcp.oh ? jcp.oh : oh_s + work_rem;
         int ih_s = nstl::max(0, -jcp.t_pad + oh_s * jcp.stride_h);
@@ -1316,7 +1350,7 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::compute_diff_weights_3d(
 
     nd_iterator_init(start, img, jcp.mb, od_s, jcp.od);
     while (start < end) {
-        auto p = jit_conv_call_s();
+        auto p = jit_conv_args_t();
         int work_rem = end - start;
         const int od_e = od_s + work_rem > jcp.od ? jcp.od : od_s + work_rem;
         int id_s = nstl::max(0, -jcp.f_pad + od_s * jcp.stride_d);
@@ -1673,7 +1707,7 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::compute_diff_weights(
     };
 
     for (int img = ti->img_start; img < ti->img_end; ++img) {
-        auto p = jit_conv_call_s();
+        auto p = jit_conv_args_t();
         if (jcp.global_transpose) {
             using simple_barrier::barrier;
             // TODO: try to call local transpositions just before jit kernel
@@ -1771,7 +1805,7 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::store_in_vnni_format(
         const int g = ti->g_start + sub_g_start;
         const int oc_b = ti->oc_b_start + sub_oc_b_start;
         const int ic_b = ti->ic_b_start + 2 * sub_icb2_start;
-        jit_conv_call_s p = jit_conv_call_s();
+        jit_conv_args_t p = jit_conv_args_t();
 
         bfloat16_t *output = (bfloat16_t *)ti->diff_weights
                 + wei_offset_ext(g, oc_b, (ic_b / 2), 0);
@@ -1934,7 +1968,7 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::prepare_scratchpad_data(
     const auto &jcp = pd()->jcp_;
 
     // XXX: See the comment about tr_iw and guarding elements in
-    // jit_avx512_core_bf16_conv_bwd_weights_kernel_f32::init_conf()
+    // jit_avx512_core_bf16_conv_bwd_weights_kernel_f32_t::init_conf()
     auto tr_src = scratchpad.template get<src_data_t>(key_conv_tr_src);
     // Zero out guard elements that cross a buffer boundary to prevent a
     // race condition due to buffer overflows from memory optimization where
