@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2023 Intel Corporation
+* Copyright 2018-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 #include <assert.h>
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
@@ -31,7 +32,7 @@
 #include "cpu/x64/jit_avx512_core_x8s8s32x_1x1_conv_kernel.hpp"
 #include "cpu/x64/jit_uni_1x1_conv_utils.hpp"
 
-#define GET_OFF(field) offsetof(jit_1x1_conv_call_s, field)
+#define GET_OFF(field) offsetof(jit_1x1_conv_args_t, field)
 
 namespace dnnl {
 namespace impl {
@@ -44,11 +45,11 @@ using namespace dnnl::impl::prop_kind;
 using namespace Xbyak;
 
 template <typename Vmm>
-_jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::
-        _jit_avx512_core_x8s8s32x_1x1_conv_kernel(
+jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::
+        jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t(
                 const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
                 const memory_desc_t &dst_md)
-    : jit_generator(jit_name())
+    : jit_generator_t(jit_name())
     , jcp(ajcp)
     , attr_(attr)
     , postops_injector_(nullptr) {
@@ -82,7 +83,7 @@ _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::bcast_loop(
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::bcast_loop(
         int load_loop_blk) {
     mov(aux1_reg_bcast_data, reg_bcast_data);
     mov(aux_reg_bcast_data, reg_bcast_data);
@@ -133,8 +134,9 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::bcast_loop(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::cvt2ps(data_type_t type_in,
-        const Vmm vmm_in, const Xbyak::Operand &op, bool mask_flag) {
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::cvt2ps(
+        data_type_t type_in, const Vmm vmm_in, const Xbyak::Operand &op,
+        bool mask_flag) {
     using namespace data_type;
     const Vmm vmm = mask_flag ? vmm_in | k_load_dim_mask | T_z : vmm_in;
     switch (type_in) {
@@ -172,7 +174,7 @@ static void iterate(const int load_loop_blk, const int ur, const F &f) {
 }
 
 template <typename Vmm>
-Address _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::output_ptr(
+Address jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::output_ptr(
         const int i_load, const int i_ur) {
     const size_t ur_stride = jcp.with_dw_conv
             ? jcp.nb_load_blocking * jcp.oc_block * i_ur
@@ -183,19 +185,19 @@ Address _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::output_ptr(
 };
 
 template <typename Vmm>
-int _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::vreg_accum_idx(
+int jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::vreg_accum_idx(
         const int load_loop_blk, int i_load, int i_ur) const {
     return (i_ur * load_loop_blk + i_load);
 };
 
 template <typename Vmm>
-Vmm _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::vreg_accum(
+Vmm jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::vreg_accum(
         const int load_loop_blk, int i_load, int i_ur) const {
     return Vmm(vreg_accum_idx(load_loop_blk, i_load, i_ur));
 };
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_sum(
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::apply_sum(
         const int load_loop_blk, const int ur, const bool mask_flag_in,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_sum) {
@@ -227,7 +229,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_sum(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_postops(
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::apply_postops(
         const int load_loop_blk, const int ur, const bool mask_flag_in,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
@@ -293,7 +295,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_postops(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::reduce_loop(
         int load_loop_blk, int ur, bool wraparound) {
     auto vreg_load = [ur, load_loop_blk](int i_load) {
         return Vmm(ur * load_loop_blk + i_load);
@@ -309,11 +311,6 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
                 reg_comp_data, sizeof(int32_t) * jcp.oc_block * i_load);
     };
 
-    auto scale_ptr = [this](int i_load) {
-        return EVEX_compress_addr(reg_ptr_scales,
-                jcp.is_oc_scale * (sizeof(float) * jcp.oc_block * i_load));
-    };
-
     auto bcast_ptr = [this](int i_reduce, int i_ur, bool bcast) {
         assert(i_ur < jcp.ur);
         assert(i_reduce <= jcp.reduce_loop_unroll);
@@ -361,7 +358,6 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
         const bool is_scale_or_zp_sum
                 = p_sum_zp_val != 0 || p_sum_scale_val != 1.f;
         mov(EVEX_compress_addr(rsp, reg_bcast_data_off), reg_bcast_data);
-        mov(reg_ptr_scales, EVEX_compress_addr(rsp, reg_ptr_sum_scale_off));
         if (is_scale_or_zp_sum) {
             mov(EVEX_compress_addr(rsp, reg_load_data_off), reg_load_data);
             if (p_sum_zp_val != 0) {
@@ -385,7 +381,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
             auto vmm_bias = vmm_tmp;
             auto vmm_comp = vmm_bcast;
             if (jcp.with_bias) {
-                if (jcp.signed_input || jcp.dst_scale)
+                if (jcp.signed_input || jcp.with_dst_scales)
                     mov(reg_bias_data,
                             EVEX_compress_addr(rsp, reg_bias_data_off));
                 cvt2ps(jcp.bia_dt, vmm_bias, bias_ptr(i_load), mask_flag);
@@ -408,33 +404,75 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
                 vcvtdq2ps(vmm_, vmm_);
             }
             for (int i_ur = 0; i_ur < ur; ++i_ur) {
-                auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-                vcvtdq2ps(r, r);
-                if (jcp.signed_input) vaddps(r, r, vmm_comp);
-                if (jcp.src_zero_point) vaddps(r, r, vmm_zp);
+                auto vmm = vreg_accum(load_loop_blk, i_load, i_ur);
+                vcvtdq2ps(vmm, vmm);
+                if (jcp.signed_input) vaddps(vmm, vmm, vmm_comp);
+                if (jcp.src_zero_point) vaddps(vmm, vmm, vmm_zp);
 
-                const Vmm mask_vmm = mask_flag ? r | k_load_dim_mask | T_z : r;
-                vmulps(mask_vmm, r, scale_ptr(i_load));
+                const Vmm vmm_k = mask_flag ? vmm | k_load_dim_mask | T_z : vmm;
 
-                if (jcp.with_bias) vaddps(r, r, vmm_bias);
+                // TODO: scales support is done not in the most optimal way.
+                // If there're two free Vmm registers, one can be used to store
+                // scale_adjust value permanently, the second one can re-use
+                // data from it and multiply by src_scale that can be obtained
+                // at the point of scales loading. Then it can be used when
+                // multiplying by wei_scales. And further re-used for dst scales
+                // to avoid reading from the same address, but reading from the
+                // Vmm instead.
+                // This would save 1st and 3rd sections for every output Vmm.
+                //
+                // If only one Vmm is found, it will add scale_adjust overhead
+                // per src_scale loading, but the second part of the idea holds.
+                //
+                // Note: attempts to identify these Vmms were not taken.
+                if (jcp.with_src_scales) {
+                    mov(reg_src_scales,
+                            EVEX_compress_addr(rsp, reg_src_scales_off));
+
+                    vmulps(vmm_k, vmm,
+                            EVEX_compress_addr(
+                                    reg_src_scales, 0, /* bcast = */ true));
+                }
+
+                if (jcp.with_wei_scales) {
+                    mov(reg_wei_scales,
+                            EVEX_compress_addr(rsp, reg_wei_scales_off));
+
+                    int scale_offset = jcp.is_oc_scale
+                            * (sizeof(float) * jcp.oc_block * i_load);
+                    vmulps(vmm_k, vmm,
+                            EVEX_compress_addr(reg_wei_scales, scale_offset,
+                                    /* bcast = */ !jcp.is_oc_scale));
+                }
+
+                if (jcp.wei_adj_scale != 1.f) {
+                    mov(reg_scale_adjust, float2int(1.f / jcp.wei_adj_scale));
+                    auto xmm_scale_adjust = Xmm(vmm_scale_adjust.getIdx());
+                    vmovq(xmm_scale_adjust, reg_scale_adjust);
+                    vbroadcastss(vmm_scale_adjust, xmm_scale_adjust);
+                    vmulps(vmm_k, vmm, vmm_scale_adjust);
+                }
+
+                if (jcp.with_bias) vaddps(vmm, vmm, vmm_bias);
             }
         }
 
         apply_postops(load_loop_blk, ur, mask_flag_in, p_sum_scale, p_sum_zp);
 
-        if (jcp.dst_scale) {
-            mov(reg_ptr_dst_scale, EVEX_compress_addr(rsp, reg_dst_scale_off));
-            vmovups(vmm_dst_scale, EVEX_compress_addr(reg_ptr_dst_scale, 0));
+        if (jcp.with_dst_scales) {
+            mov(reg_dst_scales, EVEX_compress_addr(rsp, reg_dst_scales_off));
 
             /* Apply dst scale to accumulator */
             for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
                 const bool mask_flag
                         = mask_flag_in && i_load == load_loop_blk - 1;
                 for (int i_ur = 0; i_ur < ur; ++i_ur) {
-                    const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-                    const Vmm mask_vmm
-                            = mask_flag ? r | k_load_dim_mask | T_z : r;
-                    vmulps(mask_vmm, r, vmm_dst_scale);
+                    const auto vmm = vreg_accum(load_loop_blk, i_load, i_ur);
+                    const Vmm vmm_k
+                            = mask_flag ? vmm | k_load_dim_mask | T_z : vmm;
+                    vmulps(vmm_k, vmm,
+                            EVEX_compress_addr(
+                                    reg_dst_scales, 0, /* bcast = */ true));
                 }
             }
         }
@@ -460,8 +498,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
             for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
                 for (int i_ur = 0; i_ur < ur; ++i_ur) {
                     auto r = vreg_accum(load_loop_blk, i_load, i_ur);
-                    saturate_f32(r, vmm_zero, vmm_saturation, jcp.dst_dt);
-                    vcvtps2dq(r, r);
+                    saturate_cvt_f32(r, vmm_zero, vmm_saturation, jcp.dst_dt);
                 }
             }
         }
@@ -626,7 +663,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Vmm>::generate() {
 
     preamble();
 
@@ -654,19 +691,25 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
         mov(EVEX_compress_addr(rsp, reg_src_zero_point_off),
                 reg_src_zero_point);
     }
-    if (jcp.dst_scale) {
+    if (jcp.with_src_scales) {
+        mov(reg_src_scales, ptr[param1 + GET_OFF(src_scales)]);
+        mov(EVEX_compress_addr(rsp, reg_src_scales_off), reg_src_scales);
+    }
+    if (jcp.with_wei_scales) {
+        mov(reg_wei_scales, ptr[param1 + GET_OFF(wei_scales)]);
+        mov(EVEX_compress_addr(rsp, reg_wei_scales_off), reg_wei_scales);
+    }
+    if (jcp.with_dst_scales) {
         if (!jcp.signed_input)
             mov(EVEX_compress_addr(rsp, reg_bias_data_off), reg_bias_data);
-        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
-        mov(EVEX_compress_addr(rsp, reg_dst_scale_off), reg_ptr_dst_scale);
+        mov(reg_dst_scales, ptr[param1 + GET_OFF(dst_scales)]);
+        mov(EVEX_compress_addr(rsp, reg_dst_scales_off), reg_dst_scales);
     }
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         mov(EVEX_compress_addr(rsp, reg_dst_zero_point_off),
                 reg_dst_zero_point);
     }
-    mov(reg_ptr_scales, ptr[param1 + GET_OFF(scales)]);
-    mov(EVEX_compress_addr(rsp, reg_ptr_sum_scale_off), reg_ptr_scales);
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
     mov(reg_load_data, ptr[param1 + GET_OFF(load_data)]);
     mov(reg_output_data, ptr[param1 + GET_OFF(output_data)]);
@@ -731,11 +774,11 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
         bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         if (jcp.with_bias) {
-            if (jcp.signed_input || jcp.dst_scale)
+            if (jcp.signed_input || jcp.with_dst_scales)
                 mov(reg_bias_data, EVEX_compress_addr(rsp, reg_bias_data_off));
             add(reg_bias_data,
                     load_loop_blk * jcp.load_block * jcp.typesize_bia);
-            if (jcp.signed_input || jcp.dst_scale)
+            if (jcp.signed_input || jcp.with_dst_scales)
                 mov(EVEX_compress_addr(rsp, reg_bias_data_off), reg_bias_data);
         }
         if (jcp.signed_input) {
@@ -753,11 +796,13 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
                     reg_zp_compensation);
         }
         mov(EVEX_compress_addr(rsp, reg_bcast_data_off), reg_bcast_data);
-        mov(reg_ptr_scales, EVEX_compress_addr(rsp, reg_ptr_sum_scale_off));
-        add(reg_ptr_scales,
-                jcp.is_oc_scale * load_loop_blk * jcp.load_block
-                        * sizeof(float));
-        mov(EVEX_compress_addr(rsp, reg_ptr_sum_scale_off), reg_ptr_scales);
+        if (jcp.with_wei_scales) {
+            mov(reg_wei_scales, EVEX_compress_addr(rsp, reg_wei_scales_off));
+            add(reg_wei_scales,
+                    jcp.is_oc_scale * load_loop_blk * jcp.load_block
+                            * sizeof(float));
+            mov(EVEX_compress_addr(rsp, reg_wei_scales_off), reg_wei_scales);
+        }
         mov(reg_bcast_data, EVEX_compress_addr(rsp, reg_bcast_data_off));
         add(reg_output_data, load_loop_blk * jcp.load_block * jcp.typesize_out);
         sub(reg_load_loop_work, load_loop_blk * jcp.load_loop_iter_step);
@@ -818,10 +863,11 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 }
 
-status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
+status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel_t::init_conf(
         jit_1x1_conv_conf_t &jcp, const convolution_desc_t &cd,
         const memory_desc_t *&src_md, memory_desc_t &weights_md,
         memory_desc_t &dst_md, memory_desc_t &bias_md,
@@ -838,12 +884,18 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
     const memory_desc_wrapper dst_d(&dst_md);
     const memory_desc_wrapper bias_d(&bias_md);
 
+    // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
+    // TODO: change data type of jcp fields to size_t
+    VDISPATCH_CONV_IC(!has_large_size(cd, src_d, weights_d, dst_d),
+            VERBOSE_BAD_PARAM, "large size is not supported");
+
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
-    if (!one_of(src_d.data_type(), data_type::u8, data_type::s8)
+    const bool dt_not_ok = (!one_of(src_d.data_type(), data_type::u8,
+                                    data_type::s8)
             || weights_d.data_type() != data_type::s8
             || !one_of(dst_d.data_type(), data_type::f32, data_type::s32,
-                    data_type::s8, data_type::u8, data_type::bf16))
-        return status::unimplemented;
+                    data_type::s8, data_type::u8, data_type::bf16));
+    VDISPATCH_CONV_IC(!dt_not_ok, VERBOSE_UNSUPPORTED_DT_CFG);
 
     jcp.nthr = nthreads;
 
@@ -888,8 +940,8 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
 
     jcp.os = static_cast<dim_t>(jcp.od) * jcp.oh * jcp.ow;
     jcp.is = static_cast<dim_t>(jcp.id) * jcp.ih * jcp.iw;
-
-    if (jcp.os > INT_MAX || jcp.is > INT_MAX) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            jcp.os <= INT_MAX && jcp.is <= INT_MAX, VERBOSE_LARGE_SHAPES);
 
     const auto &post_ops = attr.post_ops_;
     const int dw_conv_ind = post_ops.find(primitive_kind::convolution);
@@ -916,15 +968,16 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
         jcp.post_ops = post_ops;
     }
 
-    const auto zp = attr.zero_points_;
+    const auto &zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
     jcp.zp_src_is_common
-            = zp.common(DNNL_ARG_SRC); // otherwise, it's per-channel
+            = zp.get_mask(DNNL_ARG_SRC) == 0; // otherwise, it's per-channel
     assert(IMPLICATION(jcp.src_zero_point, jcp.zp_src_is_common));
 
-    if ((jcp.dst_zero_point || jcp.src_zero_point) && jcp.with_dw_conv)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            !((jcp.dst_zero_point || jcp.src_zero_point) && jcp.with_dw_conv),
+            VERBOSE_UNSUPPORTED_ZP_CFG);
 
     format_tag_t dat_tag = utils::pick(
             ndims - 3, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
@@ -932,7 +985,7 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
     jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag);
 
     bool args_ok = jcp.src_tag == dat_tag && jcp.dst_tag == dat_tag;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_UNSUPPORTED_TAG);
 
     if (jcp.ngroups == 1) {
         jcp.oc = rnd_up(jcp.oc, 16);
@@ -943,10 +996,17 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
     static constexpr bool sum_at_pos_0_only = false;
     static constexpr bool sum_requires_scale_one = false;
     static constexpr bool sum_requires_zp_zero = false;
-    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
+    bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
             {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
             sum_requires_scale_one, sum_requires_zp_zero));
-    if (!post_ops_ok_) return status::unimplemented;
+    // temporary workaround that skips avx512 implementation for ternary
+    // post-ops with scalar broadcasting to avoid register collisions.
+    post_ops_ok_ = post_ops_ok_
+            && IMPLICATION(jcp.with_binary,
+                    !binary_injector::
+                            any_binary_postop_rhs_with_ternary_scalar_bcast(
+                                    post_ops, dst_d));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     const int simd_w = (jcp.ic % 16 == 0 && jcp.oc % 16 == 0) ? 16
             : (jcp.ic % 8 == 0 && jcp.oc % 8 == 0)            ? 8
@@ -989,7 +1049,8 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
         return weights_md == want_wei_md;
     };
 
-    if (!set_or_check_wei_format()) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            set_or_check_wei_format(), VERBOSE_UNSUPPORTED_FORMAT_KIND);
 
     args_ok = true && jcp.oc % simd_w == 0 && jcp.ic % simd_w == 0
             && jcp.f_pad == 0 && jcp.t_pad == 0 && jcp.l_pad == 0
@@ -998,7 +1059,8 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
             && jcp.od == jcp.id && jcp.oh == jcp.ih
             && jcp.ow == jcp.iw // enforce rpad = 0
             && jcp.kd == 1 && jcp.kh == 1 && jcp.kw == 1;
-    if (!args_ok) return status::unimplemented;
+    VDISPATCH_CONV_IC(args_ok, VERBOSE_BAD_PARAM,
+            "bad config for 1x1 convolution kernel");
 
     jcp.bia_dt = jcp.with_bias ? cd.bias_desc.data_type : data_type::undef;
     jcp.dst_dt = cd.dst_desc.data_type;
@@ -1182,8 +1244,8 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
     jcp.nb_load_chunk = 1;
     // peformance improvements for googlenet_v3, mb=1;
     // TODO: generalize this condition and rewrite it in appropriate manner
-    int ncores_per_socket = (int)cpu().getNumCores(
-            Xbyak::util::IntelCpuTopologyLevel::CoreLevel);
+    int ncores_per_socket
+            = (int)cpu().getNumCores(Xbyak::util::CpuTopologyLevel::CoreLevel);
     if (jcp.mb == 1 && jcp.nb_load % 4 == 0 && jcp.ic / jcp.oc >= 4
             && jcp.ic * jcp.oc <= L2_size && jcp.nthr <= ncores_per_socket) {
         jcp.nb_load_chunk = 4;
@@ -1204,10 +1266,11 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
         jcp.nthr = nstl::min(jcp.nthr, nthr);
     }
 
-    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
-    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
-    jcp.is_oc_scale = wei_scales.mask_ != 0;
-    jcp.dst_scale = !dst_scales.has_default_values();
+    jcp.is_oc_scale = attr.scales_.get_mask(DNNL_ARG_WEIGHTS) > 0;
+    jcp.with_src_scales = !attr.scales_.get(DNNL_ARG_SRC).has_default_values();
+    jcp.with_wei_scales
+            = !attr.scales_.get(DNNL_ARG_WEIGHTS).has_default_values();
+    jcp.with_dst_scales = !attr.scales_.get(DNNL_ARG_DST).has_default_values();
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
@@ -1217,21 +1280,21 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
     return status::success;
 }
 
-void jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_scratchpad(
+void jit_avx512_core_x8s8s32x_1x1_conv_kernel_t::init_scratchpad(
         memory_tracking::registrar_t &scratchpad,
         const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr) {
     using namespace dnnl::impl::memory_tracking::names;
 
-    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
-    const dim_t scales_count
-            = wei_mask == 0 ? 1 : static_cast<dim_t>(jcp.oc) * jcp.ngroups;
-    const dim_t count = nstl::max<dim_t>(scales_count, (dim_t)jcp.ic_block);
-    scratchpad.book<float>(key_conv_adjusted_scales, count);
+    if (jcp.with_dst_scales) {
+        // See brgemm_types.hpp comment for `with_dst_scales`.
+        scratchpad.book(key_conv_dst_scales,
+                static_cast<size_t>(jcp.nthr) * sizeof(float), 4096);
+    }
 }
 
-template struct _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Xbyak::Zmm>;
-template struct _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Xbyak::Ymm>;
-template struct _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Xbyak::Xmm>;
+template struct jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Xbyak::Zmm>;
+template struct jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Xbyak::Ymm>;
+template struct jit_avx512_core_x8s8s32x_1x1_conv_kernel_vmm_t<Xbyak::Xmm>;
 
 } // namespace x64
 } // namespace cpu
