@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2024 Intel Corporation
+* Copyright 2016-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
 *******************************************************************************/
 
 #include "common/c_types_map.hpp"
+#include "common/convolution_pd.hpp"
 #include "common/memory.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
@@ -27,7 +28,7 @@
 #include "cpu/x64/injectors/jit_uni_eltwise_injector.hpp"
 #include "cpu/x64/jit_avx512_core_x8s8s32x_conv_kernel.hpp"
 
-#define GET_OFF(field) offsetof(jit_conv_call_s, field)
+#define GET_OFF(field) offsetof(jit_conv_args_t, field)
 
 namespace dnnl {
 namespace impl {
@@ -53,10 +54,10 @@ void pick_loop_order(jit_conv_conf_t &jcp, int nthr) {
 } // namespace
 
 template <typename Vmm>
-_jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::_jit_avx512_core_x8s8s32x_fwd_kernel(
-        const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
-        const memory_desc_t &dst_md)
-    : jit_generator(jit_name())
+jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::
+        jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t(const jit_conv_conf_t &ajcp,
+                const primitive_attr_t &attr, const memory_desc_t &dst_md)
+    : jit_generator_t(jit_name(), ajcp.isa)
     , jcp(ajcp)
     , attr_(attr)
     , postops_injector_(nullptr) {
@@ -93,7 +94,7 @@ _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::_jit_avx512_core_x8s8s32x_fwd_kernel(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::prepare_output(int ur_w) {
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::prepare_output(int ur_w) {
     int nb_oc_block
             = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
     for (int k = 0; k < nb_oc_block; k++)
@@ -111,20 +112,20 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::prepare_output(int ur_w) {
 }
 
 template <typename Vmm>
-Vmm _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::vmm_mask(
+Vmm jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::vmm_mask(
         const Vmm vmm_in, bool mask_flag, bool store) {
     return vmm_in;
 }
 
 template <>
-Zmm _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>::vmm_mask(
+Zmm jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Zmm>::vmm_mask(
         const Zmm zmm_in, bool mask_flag, bool store) {
     return mask_flag ? (store ? zmm_in | ktail_mask : zmm_in | ktail_mask | T_z)
                      : zmm_in;
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::cvt2ps(data_type_t type_in,
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::cvt2ps(data_type_t type_in,
         const Vmm vmm_in, const Operand &op, bool mask_flag) {
     using namespace data_type;
     const Vmm vmm = vmm_mask(vmm_in, mask_flag);
@@ -163,7 +164,7 @@ static void iterate(const int nb_oc_block, const int ur_w, const F &f) {
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_sum(int ur_w,
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::apply_sum(int ur_w,
         bool last_oc_block_flag, const int nb_oc_block, const int oc_block,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_sum) {
@@ -201,7 +202,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_sum(int ur_w,
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_postops(int ur_w,
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::apply_postops(int ur_w,
         bool last_oc_block_flag, const int nb_oc_block, const int oc_block,
         const float *p_sum_scale, const int32_t *p_sum_zp) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
@@ -242,14 +243,13 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::apply_postops(int ur_w,
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::store_output(
         int ur_w, bool last_oc_block_flag) {
     int nb_oc_block
             = jcp.is_depthwise ? jcp.nb_ch_blocking : jcp.nb_oc_blocking;
     int oc_block = jcp.is_depthwise ? jcp.ch_block : jcp.oc_block;
 
     mov(reg_bias, ptr[param1 + GET_OFF(bias)]);
-    mov(reg_ptr_scales, ptr[param1 + GET_OFF(scales)]);
     if (jcp.signed_input)
         mov(reg_compensation, ptr[param1 + GET_OFF(compensation)]);
 
@@ -270,7 +270,6 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
 
     for (int k = 0; k < nb_oc_block; k++) {
         const bool mask_flag = last_oc_block_flag && k == nb_oc_block - 1;
-        int scale_offset = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
         if (jcp.with_bias) {
             int bias_offset = jcp.typesize_bia * k * oc_block;
             auto bias_addr = EVEX_compress_addr(reg_bias, bias_offset);
@@ -305,9 +304,45 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
             if (jcp.src_zero_point) vpaddd(vmm, vmm, vmm_zp);
             vcvtdq2ps(vmm, vmm);
 
+            // TODO: scales support is done not in the most optimal way.
+            // If there're two free Vmm registers, one can be used to store
+            // scale_adjust value permanently, the second one can re-use data
+            // from it and multiply by src_scale that can be obtained at the
+            // point of scales loading. Then it can be used when multiplying
+            // by wei_scales. And further re-used for dst scales to avoid
+            // reading from the same address, but reading from the Vmm instead.
+            // This would save 1st and 3rd sections for every output Vmm.
+            //
+            // If only one Vmm is found, it will add scale_adjust overhead per
+            // src_scale loading, but the second part of the idea holds.
+            //
+            // Note: attempts to identify these Vmms were not taken.
             const Vmm vmm_k = vmm_mask(vmm, mask_flag);
-            vmulps(vmm_k, vmm,
-                    EVEX_compress_addr(reg_ptr_scales, scale_offset));
+            if (jcp.with_src_scales) {
+                mov(reg_src_scales, ptr[param1 + GET_OFF(src_scales)]);
+
+                vmulps(vmm_k, vmm,
+                        EVEX_compress_addr(
+                                reg_src_scales, 0, /* bcast = */ true));
+            }
+
+            if (jcp.with_wei_scales) {
+                mov(reg_wei_scales, ptr[param1 + GET_OFF(wei_scales)]);
+
+                int scale_offset
+                        = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
+                vmulps(vmm_k, vmm,
+                        EVEX_compress_addr(reg_wei_scales, scale_offset,
+                                /* bcast = */ !jcp.is_oc_scale));
+            }
+
+            if (jcp.wei_adj_scale != 1.f) {
+                mov(reg_scale_adjust, float2int(1.f / jcp.wei_adj_scale));
+                auto xmm_scale_adjust = Xmm(vmm_scale_adjust.getIdx());
+                vmovq(xmm_scale_adjust, reg_scale_adjust);
+                vbroadcastss(vmm_scale_adjust, xmm_scale_adjust);
+                vmulps(vmm_k, vmm, vmm_scale_adjust);
+            }
 
             if (jcp.with_bias) vaddps(vmm, vmm, vmm_bias);
         }
@@ -316,9 +351,8 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
     apply_postops(ur_w, last_oc_block_flag, nb_oc_block, oc_block, p_sum_scale,
             p_sum_zp);
 
-    if (jcp.dst_scale) {
-        mov(reg_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
-        vmovups(vmm_dst_scale, EVEX_compress_addr(reg_dst_scale, 0));
+    if (jcp.with_dst_scales) {
+        mov(reg_dst_scales, ptr[param1 + GET_OFF(dst_scales)]);
 
         /* Apply dst scale to accumulator */
         for (int k = 0; k < nb_oc_block; k++) {
@@ -326,7 +360,9 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
             for (int j = 0; j < ur_w; j++) {
                 Vmm vmm = vmm_out(j, k);
                 const Vmm vmm_k = vmm_mask(vmm, mask_flag);
-                vmulps(vmm_k, vmm, vmm_dst_scale);
+                vmulps(vmm_k, vmm,
+                        EVEX_compress_addr(
+                                reg_dst_scales, 0, /* bcast = */ true));
             }
         }
     }
@@ -351,8 +387,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
         for (int k = 0; k < nb_oc_block; k++) {
             for (int j = 0; j < ur_w; j++) {
                 Vmm vmm = vmm_out(j, k);
-                saturate_f32(vmm, vmm_zero, vmm_saturation, jcp.dst_dt);
-                vcvtps2dq(vmm, vmm);
+                saturate_cvt_f32(vmm, vmm_zero, vmm_saturation, jcp.dst_dt);
             }
         }
     }
@@ -425,13 +460,13 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker_dw(int ur_w,
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::compute_ker_dw(int ur_w,
         int pad_l, int pad_r, ic_block_t last_ic_block_flag, bool h_padded) {
     assert(!"invalid group blocking for depthwise convolution");
 }
 
 template <>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>::compute_ker_dw(int ur_w,
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Zmm>::compute_ker_dw(int ur_w,
         int pad_l, int pad_r, ic_block_t last_ic_block_flag, bool h_padded) {
 
     const bool compute_kernel = IMPLICATION(h_padded, jcp.signed_input);
@@ -492,7 +527,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>::compute_ker_dw(int ur_w,
     if (jcp.signed_input) vmovups(zmm_shifted_zero, vmm_shift);
 
     for (int ci = 0; ci < jcp.nb_ch_blocking; ci++) {
-        const bool mask_flag = last_ic_block_flag != no_last_block
+        const bool mask_flag = last_ic_block_flag != ic_block_t::no_last_block
                 && ci == jcp.nb_ch_blocking - 1;
         if (jcp.is_resrc_depthwise && !h_padded) {
             // now we can load input once and reuse up to jcp.kw times
@@ -592,8 +627,8 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>::compute_ker_dw(int ur_w,
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
-        int pad_r, ic_block_t last_ic_block_flag, bool h_padded) {
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::compute_ker(int ur_w,
+        int pad_l, int pad_r, ic_block_t last_ic_block_flag, bool h_padded) {
     if (jcp.is_depthwise)
         return compute_ker_dw(ur_w, pad_l, pad_r, last_ic_block_flag, h_padded);
 
@@ -645,7 +680,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
         int _end = jcp.signed_input ? ur_w : jj_end;
         /* Skip the last loads of input
             if (ic%16)/ic_sub_step < ic_block/ic_sub_step */
-        int icb = (last_ic_block_flag != no_last_block)
+        int icb = (last_ic_block_flag != ic_block_t::no_last_block)
                 ? div_up((jcp.ic_without_padding % ic_block), ic_sub_step)
                 : ic_block / ic_sub_step;
         if (compute_kernel) {
@@ -660,7 +695,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
                     for (int jj = _start; jj < _end; jj++) {
                         int aux_input_offset = input_offset(jj, ic, ki);
                         if (jj >= jj_start && jj < jj_end) {
-                            if (last_ic_block_flag == last_sp_block
+                            if (last_ic_block_flag == ic_block_t::last_sp_block
                                     && ic_tail_size != 0 && ic == icb - 1) {
                                 Xmm xmm_tmp = Xmm(
                                         vmm_inp(jj, nb_oc_block).getIdx());
@@ -734,7 +769,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::compute_ker(int ur_w, int pad_l,
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::kh_loop(
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::kh_loop(
         int ur_w, int pad_l, int pad_r, ic_block_t last_ic_block_flag) {
     Label kd_label, kh_label, skip_kd_loop, skip_kh_loop;
     Label f_overflow_label, no_f_overflow_label, d_h_f_overflow_label,
@@ -884,7 +919,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::kh_loop(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::icb_loop(
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::icb_loop(
         int ur_w, int pad_l, int pad_r, bool is_last_sp_block) {
 
     if (jcp.src_zero_point && !jcp.is_depthwise) {
@@ -912,17 +947,18 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::icb_loop(
             jne(common_ker, T_NEAR);
         }
         kh_loop(ur_w, pad_l, pad_r,
-                is_last_sp_block ? last_sp_block : last_ic_block);
+                is_last_sp_block ? ic_block_t::last_sp_block
+                                 : ic_block_t::last_ic_block);
         if (do_icb_loop) {
             jmp(end_ker, T_NEAR);
 
             L(common_ker);
-            kh_loop(ur_w, pad_l, pad_r, no_last_block);
+            kh_loop(ur_w, pad_l, pad_r, ic_block_t::no_last_block);
 
             L(end_ker);
         }
     } else {
-        kh_loop(ur_w, pad_l, pad_r, no_last_block);
+        kh_loop(ur_w, pad_l, pad_r, ic_block_t::no_last_block);
     }
     // End of IC Loop
     if (do_icb_loop) {
@@ -964,7 +1000,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::icb_loop(
 }
 
 template <typename Vmm>
-void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
+void jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Vmm>::generate() {
     Label permute_index_table;
     int in_ic_shift = jcp.is_fused_conv ? jcp.dw_conv_buffer_oc
                                         : jcp.ic_without_padding * jcp.ngroups;
@@ -1147,7 +1183,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
         r_pad_fall_through_n_urw = (cur_ow / jcp.ur_w) % n_urw_per_ow_block;
         r_pad_fall_through_ow_block = cur_ow / (n_urw_per_ow_block * jcp.ur_w);
 
-        // r_pad or last_sp_block
+        // r_pad or ic_block_t::last_sp_block
         if (cur_ow + jcp.ur_w <= jcp.ow) {
             if (r_pad_fall_through_n_urw == 0) ++n_labels;
             const int n_urw_r_pad_region = (jcp.ow - cur_ow) / jcp.ur_w;
@@ -1286,7 +1322,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
         }
     }
 
-    // r_pad region or last_sp_block
+    // r_pad region or ic_block_t::last_sp_block
     if (cur_ow + jcp.ur_w <= jcp.ow) {
         if (jcp.nb_ow > 1) {
             if (cur_n_oi == 0) {
@@ -1345,7 +1381,8 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
 
     postamble();
 
-    if (jcp.with_eltwise) postops_injector_->prepare_table();
+    if (jcp.with_eltwise)
+        postops_injector_->prepare_table(/* generate = */ true);
 
     if (jcp.is_fast_depthwise) {
         align(64);
@@ -1357,7 +1394,7 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::generate() {
     }
 }
 
-status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
+status_t jit_avx512_core_x8s8s32x_fwd_kernel_t::init_conf(jit_conv_conf_t &jcp,
         const convolution_desc_t &cd, memory_desc_t &src_md,
         memory_desc_t &weights_md, memory_desc_t &dst_md,
         memory_desc_t &bias_md, primitive_attr_t &attr, int nthreads) {
@@ -1368,6 +1405,11 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     const memory_desc_wrapper dst_d(&dst_md);
     const memory_desc_wrapper bias_d(&bias_md);
 
+    // Big int (> INT_MAX) values are unsupported and jcp fields may overflow
+    // TODO: change data type of jcp fields to size_t
+    VDISPATCH_CONV_IC(!has_large_size(cd, src_d, weights_d, dst_d),
+            VERBOSE_BAD_PARAM, "large size is not supported");
+
     const bool with_groups = weights_d.ndims() == src_d.ndims() + 1;
     const int ndims = src_d.ndims();
     const bool is_1d = ndims == 3;
@@ -1375,12 +1417,13 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     const bool is_3d = ndims == 5;
     assert(is_1d || is_2d || is_3d);
 
-    if (!(mayiuse(avx512_core)
-                && one_of(src_d.data_type(), data_type::u8, data_type::s8)
-                && weights_d.data_type() == data_type::s8
-                && one_of(dst_d.data_type(), data_type::f32, data_type::s32,
-                        data_type::s8, data_type::u8, data_type::bf16)))
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            (mayiuse(avx512_core)
+                    && one_of(src_d.data_type(), data_type::u8, data_type::s8)
+                    && weights_d.data_type() == data_type::s8
+                    && one_of(dst_d.data_type(), data_type::f32, data_type::s32,
+                            data_type::s8, data_type::u8, data_type::bf16)),
+            VERBOSE_ISA_DT_MISMATCH);
 
     jcp = zero<decltype(jcp)>();
     jcp.nthr = nthreads;
@@ -1429,12 +1472,20 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     jcp.is_depthwise = true && with_groups && everyone_is(1, jcp.ic, jcp.oc);
 
     // Used for bfloat16 output
-    jcp.isa = mayiuse(avx512_core_bf16) ? avx512_core_bf16
-                                        : bf16_emulation_t::get_isa();
+    jcp.isa = mayiuse(avx512_core_vnni) ? avx512_core_vnni : avx512_core;
+    if (dst_d.data_type() == bf16) {
+        if (mayiuse(avx512_core_bf16)) {
+            jcp.isa = avx512_core_bf16;
+        } else {
+            const auto bf16_emulation_isa = bf16_emulation_t::get_isa();
+            VDISPATCH_CONV_IC(is_superset(jcp.isa, bf16_emulation_isa),
+                    VERBOSE_UNSUPPORTED_ISA);
+        }
+    }
 
-    if (jcp.is_depthwise && is_3d)
-        // NOTE: 3D depthwise is not currently supported here.
-        return status::unimplemented;
+    // NOTE: 3D depthwise is not currently supported here.
+    VDISPATCH_CONV_IC(!(jcp.is_depthwise && is_3d), VERBOSE_UNSUPPORTED_FEATURE,
+            "does not support depthwise convolution");
 
     if (jcp.is_depthwise) {
         jcp.ch_block = 16;
@@ -1460,26 +1511,29 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
             jcp.ic_block = (jcp.ic % 8 == 0) && (jcp.oc % 8 == 0) ? 8 : 4;
             jcp.oc_block = jcp.ic_block;
         }
-        if (jcp.ic % jcp.ic_block != 0 || jcp.oc % jcp.oc_block != 0)
-            return status::unimplemented;
+        VDISPATCH_CONV_IC(
+                (jcp.ic % jcp.ic_block == 0 && jcp.oc % jcp.oc_block == 0),
+                VERBOSE_BLOCKING_FAIL, "bad blocking dimensions");
     }
 
     jcp.simd_w = jcp.is_depthwise ? jcp.ch_block : jcp.ic_block;
 
-    const auto zp = attr.zero_points_;
+    const auto &zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
     jcp.zp_src_is_common
-            = zp.common(DNNL_ARG_SRC); // otherwise, it's per-channel
+            = zp.get_mask(DNNL_ARG_SRC) == 0; // otherwise, it's per-channel
     assert(IMPLICATION(jcp.src_zero_point, jcp.zp_src_is_common));
 
-    if ((jcp.dst_zero_point || jcp.src_zero_point) && jcp.is_fused_conv)
-        return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            !((jcp.dst_zero_point || jcp.src_zero_point) && jcp.is_fused_conv),
+            VERBOSE_UNSUPPORTED_ZP_CFG);
 
-    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
-    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
-    jcp.is_oc_scale = wei_scales.mask_ != 0;
-    jcp.dst_scale = !dst_scales.has_default_values();
+    jcp.is_oc_scale = attr.scales_.get_mask(DNNL_ARG_WEIGHTS) > 0;
+    jcp.with_src_scales = !attr.scales_.get(DNNL_ARG_SRC).has_default_values();
+    jcp.with_wei_scales
+            = !attr.scales_.get(DNNL_ARG_WEIGHTS).has_default_values();
+    jcp.with_dst_scales = !attr.scales_.get(DNNL_ARG_DST).has_default_values();
 
     jcp.has_vnni = mayiuse(avx512_core_vnni);
     const bool bf16_req_extra_regs = cd.dst_desc.data_type == data_type::bf16
@@ -1554,7 +1608,8 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
         return weights_md == want_wei_md;
     };
 
-    if (!set_or_check_wei_format()) return status::unimplemented;
+    VDISPATCH_CONV_IC(
+            set_or_check_wei_format(), VERBOSE_UNSUPPORTED_FORMAT_KIND);
 
     format_tag_t dat_tag = utils::pick(
             ndims - 3, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
@@ -1565,7 +1620,7 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     } else {
         jcp.src_tag = src_d.matches_one_of_tag(dat_tag);
     }
-    if (jcp.src_tag != dat_tag) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.src_tag == dat_tag, VERBOSE_UNSUPPORTED_TAG_S, "src");
 
     if (dst_d.format_kind() == format_kind::any) {
         CHECK(memory_desc_init_by_tag(dst_md, dat_tag));
@@ -1573,7 +1628,7 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     } else {
         jcp.dst_tag = dst_d.matches_one_of_tag(dat_tag);
     }
-    if (jcp.dst_tag != dat_tag) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.dst_tag == dat_tag, VERBOSE_UNSUPPORTED_TAG_S, "dst");
 
     if (jcp.with_bias) {
         if (bias_d.format_kind() == format_kind::any)
@@ -1602,10 +1657,17 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     static constexpr bool sum_at_pos_0_only = false;
     static constexpr bool sum_requires_scale_one = false;
     static constexpr bool sum_requires_zp_zero = false;
-    const bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
+    bool post_ops_ok_ = post_ops_ok(post_ops_ok_args_t(avx512_core,
             {eltwise, binary, sum}, jcp.post_ops, &dst_d, sum_at_pos_0_only,
             sum_requires_scale_one, sum_requires_zp_zero));
-    if (!post_ops_ok_) return status::unimplemented;
+    // temporary workaround that skips avx512 implementation for ternary
+    // post-ops with scalar broadcasting to avoid register collisions.
+    post_ops_ok_ = post_ops_ok_
+            && IMPLICATION(jcp.with_binary,
+                    !binary_injector::
+                            any_binary_postop_rhs_with_ternary_scalar_bcast(
+                                    post_ops, dst_d));
+    VDISPATCH_CONV_IC(post_ops_ok_, VERBOSE_UNSUPPORTED_POSTOP);
 
     jcp.typesize_in = types::data_type_size(src_d.data_type());
     jcp.typesize_out = types::data_type_size(dst_d.data_type());
@@ -1636,8 +1698,8 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     int max_threading_nb_oc_chunk = 4;
     // Performance improvements for googlenet_v3 and resnet_50 with mb = 1;
     // TODO: generalize this condition and rewrite it in appropriate manner
-    int ncores_per_socket = (int)cpu().getNumCores(
-            Xbyak::util::IntelCpuTopologyLevel::CoreLevel);
+    int ncores_per_socket
+            = (int)cpu().getNumCores(Xbyak::util::CpuTopologyLevel::CoreLevel);
     if (jcp.has_vnni && jcp.mb == 1 && jcp.kh == 3 && jcp.kw == 3
             && jcp.stride_w == 1 && jcp.ic % 64 == 0
             && jcp.nthr <= ncores_per_socket)
@@ -1714,10 +1776,10 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
      * the threshold L1_cache_size is empirical */
     size_t wei_size
             = sizeof(float) * jcp.ic * jcp.oc * jcp.kh * jcp.kw * jcp.kd;
-    size_t out_size
-            = jcp.mb * jcp.typesize_out * jcp.oc * jcp.oh * jcp.ow * jcp.od;
-    size_t inp_size
-            = jcp.mb * jcp.typesize_in * jcp.ic * jcp.ih * jcp.iw * jcp.id;
+    size_t out_size = static_cast<size_t>(jcp.mb) * jcp.typesize_out * jcp.oc
+            * jcp.oh * jcp.ow * jcp.od;
+    size_t inp_size = static_cast<size_t>(jcp.mb) * jcp.typesize_in * jcp.ic
+            * jcp.ih * jcp.iw * jcp.id;
     size_t total_size = jcp.ngroups * (wei_size + out_size + inp_size);
     const unsigned int L1_cache_size = platform::get_per_core_cache_size(1);
 
@@ -1740,7 +1802,8 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
         }
     }
 
-    if (jcp.oc % jcp.oc_block != 0) return status::unimplemented;
+    VDISPATCH_CONV_IC(jcp.oc % jcp.oc_block == 0, VERBOSE_BLOCKING_FAIL,
+            "block size does not divide output channel");
 
     pick_loop_order(jcp, jcp.nthr);
 
@@ -1754,18 +1817,21 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     return status::success;
 }
 
-void jit_avx512_core_x8s8s32x_fwd_kernel::init_scratchpad(
+void jit_avx512_core_x8s8s32x_fwd_kernel_t::init_scratchpad(
         memory_tracking::registrar_t &scratchpad, const jit_conv_conf_t &jcp,
         const primitive_attr_t &attr) {
-    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
-    const dim_t scales_count = wei_mask == 0 ? 1 : jcp.oc * jcp.ngroups;
-    dim_t count = wei_mask == 0 ? (dim_t)16 : scales_count;
-    scratchpad.book<float>(key_conv_adjusted_scales, count);
+    if (jcp.with_dst_scales) {
+        const size_t n_dst_scales = static_cast<size_t>(
+                jcp.is_depthwise ? jcp.ngroups : jcp.nthr);
+        // See brgemm_types.hpp comment for `with_dst_scales`.
+        scratchpad.book(
+                key_conv_dst_scales, n_dst_scales * sizeof(float), 4096);
+    }
 }
 
-template struct _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>;
-template struct _jit_avx512_core_x8s8s32x_fwd_kernel<Ymm>;
-template struct _jit_avx512_core_x8s8s32x_fwd_kernel<Xmm>;
+template struct jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Zmm>;
+template struct jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Ymm>;
+template struct jit_avx512_core_x8s8s32x_fwd_kernel_vmm_t<Xmm>;
 } // namespace x64
 } // namespace cpu
 } // namespace impl
