@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2024 Intel Corporation
+* Copyright 2021-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,7 +21,6 @@
 #include "common/utils.hpp"
 
 #include "cpu/cpu_primitive.hpp"
-#include "cpu/scale_utils.hpp"
 
 #include "cpu/x64/amx_tile_configure.hpp"
 #include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
@@ -51,11 +50,16 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
     const auto wei_type = weights_md(0)->data_type;
     const auto dst_type = dst_md(0)->data_type;
     const bool is_int8 = one_of(src_type, u8, s8);
+    const bool is_fp8 = one_of(src_type, f8_e4m3, f8_e5m2);
+
+    VDISPATCH_CONV(
+            impl::is_dense_format_kind({src_md(), weights_md(), dst_md()}),
+            VERBOSE_UNSUPPORTED_SPARSE_CFG);
 
     using skip_mask_t = primitive_attr_t::skip_mask_t;
     auto skip_mask = skip_mask_t::post_ops | skip_mask_t::sum_dt
-            | skip_mask_t::zero_points_runtime | skip_mask_t::fpmath_mode;
-    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::scales_runtime;
+            | skip_mask_t::zero_points | skip_mask_t::fpmath_mode;
+    if (is_int8 || is_fp8) skip_mask |= skip_mask_t::scales;
 
     VDISPATCH_CONV(is_fwd(), VERBOSE_BAD_PROPKIND);
     VDISPATCH_CONV(expect_data_types(src_type, wei_type, data_type::undef,
@@ -76,17 +80,22 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
             VERBOSE_UNSUPPORTED_ATTR);
     VDISPATCH_CONV(attr()->post_ops_.check_sum_consistency(dst_type, is_int8),
             VERBOSE_UNSUPPORTED_POSTOP);
-    VDISPATCH_CONV(zero_points_ok(), VERBOSE_UNSUPPORTED_ZP_CFG);
-    VDISPATCH_CONV(arg_scales_ok(), VERBOSE_UNSUPPORTED_SCALES_CFG);
+    CHECK(attr_scales_ok());
+    CHECK(attr_zero_points_ok());
 
     CHECK(brgemm_convolution_utils::init_1x1_conf(jcp_, isa, *desc(), src_md_,
             weights_md_, dst_md_, bias_md_, attr_, dnnl_get_max_threads()));
 
     brgs_ = std::make_shared<brgemm_containers::brgemm_desc_container_t>(32);
 
+    const bool need_compensation
+            = (jcp_.src_zero_point || jcp_.s8s8_compensation_required)
+            && !jcp_.req_brg_comp_pad;
+
     ic_chunks_ = div_up(jcp_.nb_ic, jcp_.nb_ic_blocking);
     need_postwork_ = jcp_.with_bias || jcp_.with_eltwise || jcp_.with_binary
-            || (one_of(src_type, u8, s8) && wei_type == s8) // oscales needed
+            || jcp_.with_src_scales || jcp_.with_wei_scales
+            || jcp_.with_dst_scales || need_compensation
             || (jcp_.dst_dt != jcp_.acc_dt) || jcp_.with_sum;
 
     const bool need_extra_m_kernel = get_extra_m_kernel_req(jcp_);
@@ -102,7 +111,8 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
         if (vM == 0 || vN == 0 || vK == 0) continue;
 
         if (!jcp_.is_reduced_rtus) {
-            brgemm_init_params_.emplace_front(i_init, vM, vN, vK, jcp_.LDA);
+            brgemm_init_params_.emplace_front(
+                    i_init, vM, vN, vK, jcp_.LDA, jcp_.extendable_k);
         } else {
             const bool is_accum_kernel = i_init == 0;
 
@@ -119,7 +129,8 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
             const auto brgemm_K = use_rtus_K ? rtus_k : vK;
             const bool use_rtus_LDA = use_rtus_K && is_accum_kernel;
             const auto LDA = use_rtus_LDA ? jcp_.rtus_padded_ic_size : jcp_.LDA;
-            brgemm_init_params_.emplace_front(i_init, vM, vN, brgemm_K, LDA);
+            brgemm_init_params_.emplace_front(
+                    i_init, vM, vN, brgemm_K, LDA, false);
         }
     }
 
@@ -134,12 +145,12 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
             auto vK = is_accum_kernel
                     ? jcp_.rtus_ic_size
                     : jcp_.ic_without_padding - jcp_.rtus_ic_size;
+            if (vM <= 0 || vK <= 0) continue;
             const bool use_rtus_LDA = is_accum_kernel;
             const auto LDA = use_rtus_LDA ? jcp_.rtus_padded_ic_size : jcp_.LDA;
             constexpr int extra_m_kernel_start_idx = 2;
             brgemm_init_params_.emplace_front(
-                    extra_m_kernel_start_idx + idx, vM, vN, vK, LDA);
-            assert(vM > 0 && vK > 0);
+                    extra_m_kernel_start_idx + idx, vM, vN, vK, LDA, false);
         }
     }
 
@@ -148,9 +159,6 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
     brgemm_convolution_utils::set_amx_wsp_per_thread(jcp_);
     auto scratchpad = scratchpad_registry().registrar();
     brgemm_convolution_utils::init_scratchpad(scratchpad, jcp_);
-    if (jcp_.with_scales)
-        book_precomputed_scales(scratchpad, attr()->scales_, OC(),
-                jcp_.scale_adjust_factor != 1.0f);
 
     return status::success;
 }
@@ -175,7 +183,7 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init_brgemm_desc() {
 
         const auto brg_idx = get_brg_idx(jcp_, params);
 
-        brgemm_t brg;
+        brgemm_desc_t brg;
         brgemm_strides_t brg_strides;
         brg_strides.stride_a = jcp_.brg_stride_a;
         brg_strides.stride_b = jcp_.brg_stride_b;
@@ -183,7 +191,7 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init_brgemm_desc() {
                 = (jcp_.brg_type == brgemm_strd) ? &brg_strides : nullptr;
         CHECK(brgemm_desc_init(&brg, isa, jcp_.brg_type, src_type, wei_type,
                 false, false, brgemm_row_major, alpha, vbeta, LDA, jcp_.LDB,
-                jcp_.LDC, vM, vN, vK, strides_ptr));
+                jcp_.LDC, vM, vN, vK, strides_ptr, jcp_.is_tf32));
 
         brgemm_attr_t brgattr;
         brgattr.max_bs = jcp_.gemm_batch_size;
@@ -200,7 +208,8 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init_brgemm_desc() {
         brgattr.hint_expected_B_size = vN * vK;
         brgattr.hint_expected_C_size = bd_blocking * vN;
 
-        brgattr.wary_tail_read = false;
+        brgattr.wary_A_k_tail_read = params.wary_tail_read_;
+        brgattr.extendable_k = jcp_.extendable_k;
         brgattr.use_uker = jcp_.use_uker;
         brgattr.use_interleave_stores = jcp_.use_interleave_stores;
         brgattr.hint_prefetching = jcp_.hint_prefetching;
@@ -218,6 +227,8 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init_brgemm_desc() {
         brg.with_weights_scale_adjust = jcp_.scale_adjust_factor != 1.0f;
         CHECK(brgemm_desc_set_postops(
                 &brg, attr(), &dst_md_, LDD, jcp_.bia_dt));
+        CHECK(brgemm_desc_finalize(&brg));
+
         jcp_.amx_buf_size_per_thread = nstl::max(
                 brg.get_wsp_buffer_size(), jcp_.amx_buf_size_per_thread);
         brgs_->insert(brg_idx, brg);
@@ -258,15 +269,14 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::init(engine_t *engine) {
     dst_d_sz = OD * dst_h_sz;
 
     const auto src_type = pd()->src_md(0)->data_type;
-
-    const auto last_ic_block = (isa == avx512_core_fp16 && src_type == f16)
-            ? 1
-            : data_type_vnni_granularity(src_type);
+    const data_type_t last_ic_block_dt
+            = get_mac_emu_data_type(src_type, isa, isa == avx512_core_fp16);
+    const auto last_ic_block = data_type_vnni_granularity(last_ic_block_dt);
 
     wei_ic_stride = jcp.wei_plain ? jcp.oc_without_padding : jcp.oc_block;
     wei_ocb_stride = jcp.wei_plain
             ? jcp.oc_block
-            : (dim_t)rnd_up(jcp.ic, last_ic_block) * jcp.oc_block;
+            : (dim_t)rnd_up(jcp.icp, last_ic_block) * jcp.oc_block;
     wei_g_stride = jcp.wei_plain ? jcp.oc : jcp.nb_oc * wei_ocb_stride;
 
     if (jcp.is_rtus) {
@@ -276,20 +286,6 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::init(engine_t *engine) {
         CHECK(rtus_kernel_->create_kernel());
     }
 
-    // JIT to precompute scales
-    const bool is_jit_supported = mayiuse(avx512_core);
-    const auto attr = pd()->attr();
-    if (is_jit_supported && req_copy_scales(attr, jcp.scale_adjust_factor)) {
-        const auto &attr_scales = attr->scales_;
-        int wei_scale_mask = attr_scales.get(DNNL_ARG_WEIGHTS).mask_;
-        if (wei_scale_mask != 0) {
-            CHECK(safe_ptr_assign(jit_scale_precompute_,
-                    new jit_avx512_core_scale_precompute_t(
-                            jcp.scale_adjust_factor)));
-            CHECK(jit_scale_precompute_->create_kernel());
-        }
-    }
-
     for (auto &params : pd()->brgemm_init_params_) {
         const auto brg_idx = get_brg_idx(jcp, params);
         const auto &brgs = *(pd()->brgs_);
@@ -333,16 +329,22 @@ void brgemm_1x1_convolution_fwd_t<isa>::maybe_rtus(int ithr,
             : icc * jcp.nb_ic_blocking * jcp.ic_block;
     const auto g_ic = g * jcp.ic_without_padding + icc_tail_start;
 
+    const memory_desc_wrapper src_d(pd()->src_md());
+
     auto call_kernel = [&](int nh, int nw, int od, int oh, int ow) {
         assert(nh == 0 || (nw == 0 && ow == 0));
         if (utils::everyone_is(0, nh, nw)) return;
         const int id = od * jcp.stride_d;
         const int ih = oh * jcp.stride_h;
         const int iw = ow * jcp.stride_w;
-        const auto inp_offset = n * src_d_sz + id * src_h_sz + ih * src_w_sz
-                + iw * jcp.ngroups * jcp.ic_without_padding + g_ic;
+
+        // Using blk_off to offset batch is motivated input\output striding aligment
+        const auto inp_offset = src_d.off_l(0)
+                + n * src_d.blk_off<false, true>(1) + id * src_h_sz
+                + ih * src_w_sz + iw * jcp.ngroups * jcp.ic_without_padding
+                + g_ic;
         auto p = jit_avx512_core_brgemm_conv_trans_kernel::
-                jit_brgemm_conv_trans_kernel_call_s();
+                jit_brgemm_conv_trans_kernel_args_t();
         p.h_count = nh;
         p.owb = nw;
         p.src = src + src_dt_size * inp_offset;
@@ -391,9 +393,10 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
         brgemm_batch_element_t *const __restrict brg_batch,
         char *const c_buffer, const char *inp_buffer, int g, int n, int ocb,
         int od, int oh, int ow, int icc, int *last_brg_idx,
-        const float *oscales, int32_t src_zp_vals, int32_t *src_zp_comp,
-        int32_t *dst_zp_vals, int32_t *s8s8_compensation,
-        const float *dst_scales, const bool is_last_os) const {
+        const int32_t *src_zero_points, int32_t *src_zp_comp,
+        const int32_t *dst_zero_points, int32_t *s8s8_compensation,
+        const void *src_scales, const void *wei_scales, const void *dst_scales,
+        const bool is_last_os) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper weights_d(pd()->weights_md());
@@ -426,7 +429,6 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
 
     const int icb = icc * jcp.nb_ic_blocking;
     const int ic = icb * jcp.ic_block;
-    const int g_ic = g * jcp.ic + ic;
 
     const bool use_special_m_idx = get_extra_m_kernel_req(jcp) && is_last_os;
     const int kernel_init = static_cast<int>(icc == 0) + 2 * use_special_m_idx;
@@ -441,19 +443,33 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
             : (icc == pd()->ic_chunks_ - 1
                     && ((jcp.ic - ic) % jcp.ic_block != 0));
 
-    const auto src_offset = n * src_d_sz + id * src_h_sz + ih * src_w_sz
-            + iw * jcp.ngroups * jcp.ic_without_padding + g_ic;
-    const auto rtus_src
-            = jcp.is_reduced_rtus ? src + src_dt_size * src_offset : inp_buffer;
+    // Using blk_off to offset batch is motivated input\output striding aligment
+    // See `blk_off` definition.
+    const auto src_mb_c_offset = src_dt_size
+            * (src_d.off_l(0) + n * src_d.blk_off<false, true>(1)
+                    + g * src_d.blk_off<false, true>(0, 1) * jcp.ic + ic);
+    const auto src_hw_offset = src_dt_size
+            * (id * src_h_sz + ih * src_w_sz
+                    + iw * jcp.ngroups * jcp.ic_without_padding);
+
+    const auto rtus_src = jcp.is_reduced_rtus
+            ? src + src_mb_c_offset + src_hw_offset
+            : inp_buffer;
     const auto src_base
-            = jcp.is_rtus ? rtus_src : src + src_dt_size * src_offset;
+            = jcp.is_rtus ? rtus_src : src + src_mb_c_offset + src_hw_offset;
 
     const auto wei_offset = g * wei_g_stride + ocb * wei_ocb_stride;
     const auto wei_base = weights + wei_dt_size * wei_offset;
-    const auto ptr_D = dst
-            + dst_dt_size
-                    * (n * dst_d_sz + od * dst_h_sz + oh * dst_w_sz
-                            + ow * jcp.oc_without_padding + g_oc);
+
+    // Using blk_off to offset batch is motivated input\output striding aligment
+    // See `blk_off` definition.
+    const auto dst_base = dst_dt_size
+            * (dst_d.off_l(0) + n * dst_d.blk_off<false, true>(1)
+                    + g * dst_d.blk_off<false, true>(0, 1) * jcp.oc + oc);
+    const auto dst_offset = dst_dt_size
+            * (od * dst_h_sz + oh * dst_w_sz + ow * jcp.oc_without_padding);
+
+    const auto ptr_D = dst + dst_base + dst_offset;
     char *const ptr_C = (jcp.use_buffer) ? c_buffer : (char *)ptr_D;
 
     const auto bias_w
@@ -471,6 +487,8 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
             ? &s8s8_compensation[comp_offset]
             : nullptr;
 
+    const bool wary_tail_read = jcp.extendable_k;
+
     const auto call_brgemm = [&](int brg_idx, int ic_block_s, int n_ic_blocks,
                                      bool do_postops, bool brgemm_is_ic_tail) {
         // NOTE: avoid some costly tile reconfigurations here by keeping track
@@ -499,14 +517,18 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
 
         const auto brg_ker = brg_kernels_[brg_idx];
         if (do_postops) {
+            const int32_t src_zp_val = src_zero_points ? src_zero_points[0] : 0;
             const brgemm_post_ops_data_t post_ops_data {
                     static_cast<const void *>(bias_w),
-                    &oscales[jcp.is_oc_scale * g_oc],
                     post_ops_binary_rhs_arg_vec.data(),
                     static_cast<size_t>(g_oc), 0, dst, 0,
                     static_cast<void *>(src_zp_comp_ptr), nullptr,
-                    static_cast<void *>(dst_zp_vals), false, src_zp_vals, false,
-                    false, dst_scales};
+                    dst_zero_points, false, src_zp_val, false, false,
+                    src_scales,
+                    wei_scales ? static_cast<const char *>(wei_scales)
+                                    + jcp.is_oc_scale * g_oc * sizeof(float)
+                               : nullptr,
+                    dst_scales};
 
             void *scratch = is_amx ? static_cast<void *>(wsp_tile)
                                    : static_cast<void *>(s8s8_comp_ptr);
@@ -522,18 +544,21 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
 
     const auto do_post_work = (pd()->need_postwork_ || jcp.use_buffer)
             && icc == pd()->ic_chunks_ - 1;
-    if (jcp.is_reduced_rtus || nb_ic_b > 0) {
-        const auto brg_idx
-                = get_brg_idx(kernel_init, is_os_tail, is_oc_tail, false);
-        call_brgemm(brg_idx, 0, jcp.is_reduced_rtus ? 1 : nb_ic_b,
-                do_post_work && !is_ic_tail, false);
+    if (jcp.is_reduced_rtus) {
+        const auto brg_idx = get_brg_idx(
+                kernel_init, is_os_tail, is_oc_tail, false, wary_tail_read);
+        call_brgemm(brg_idx, 0, 1, do_post_work && !is_ic_tail, false);
+    } else if (nb_ic_b > 0) {
+        const auto brg_idx = get_brg_idx(
+                kernel_init, is_os_tail, is_oc_tail, false, wary_tail_read);
+        call_brgemm(brg_idx, 0, nb_ic_b, do_post_work && !is_ic_tail, false);
     }
     if (is_ic_tail) {
         const auto use_init_ker = jcp.is_reduced_rtus
                 ? kernel_init - 1
                 : kernel_init && (nb_ic_b == 0);
-        const auto brg_idx = get_brg_idx(
-                use_init_ker, is_os_tail, is_oc_tail, !jcp.is_reduced_rtus);
+        const auto brg_idx = get_brg_idx(use_init_ker, is_os_tail, is_oc_tail,
+                !jcp.is_reduced_rtus, wary_tail_read);
         call_brgemm(brg_idx, jcp.is_reduced_rtus ? 0 : nb_ic_b, 1, do_post_work,
                 jcp.is_reduced_rtus);
     }
@@ -542,9 +567,10 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
 template <cpu_isa_t isa>
 void brgemm_1x1_convolution_fwd_t<isa>::execute_os_blocking(
         const brgemm_exec_ctx_t &brgemm_ctx,
-        brgemm_batch_element_t *const brg_batch_global, const float *dst_scales,
-        const float *oscales, int32_t src_zp_vals, int32_t *src_zp_comp,
-        int32_t *dst_zp_vals, int32_t *s8s8_compensation,
+        brgemm_batch_element_t *const brg_batch_global, const void *src_scales,
+        const void *wei_scales, const void *dst_scales, void *dst_scales_inv,
+        const int32_t *src_zero_points, int32_t *src_zp_comp,
+        const int32_t *dst_zero_points, int32_t *s8s8_compensation,
         char *const c_buffer_global, char *inp_buffer_base,
         uint8_t *inp_buffer_mask_base) const {
 
@@ -567,6 +593,15 @@ void brgemm_1x1_convolution_fwd_t<isa>::execute_os_blocking(
         uint8_t *__restrict inp_buffer_mask = (jcp.is_rtus)
                 ? inp_buffer_mask_base + ithr * jcp.inp_buffer_mask_size
                 : nullptr;
+
+        float *dst_scales_inv_ptr = nullptr;
+        if (jcp.with_dst_scales) {
+            const float *dst_scales_ptr
+                    = static_cast<const float *>(dst_scales);
+            dst_scales_inv_ptr = static_cast<float *>(dst_scales_inv) + ithr;
+            dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+        }
+
         int last_n = -1;
         int last_g = -1;
         int last_brg_idx = -1;
@@ -605,9 +640,9 @@ void brgemm_1x1_convolution_fwd_t<isa>::execute_os_blocking(
                     const bool is_last_os = (osb_start + osb) == jcp.nb_os - 1;
                     exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer,
                             inp_buffer_sp, g, n, ocb, od, oh, ow, icc,
-                            &last_brg_idx, oscales, src_zp_vals, src_zp_comp,
-                            dst_zp_vals, s8s8_compensation, dst_scales,
-                            is_last_os);
+                            &last_brg_idx, src_zero_points, src_zp_comp,
+                            dst_zero_points, s8s8_compensation, src_scales,
+                            wei_scales, dst_scales_inv_ptr, is_last_os);
                 }
             }
             last_n = n;
@@ -628,9 +663,10 @@ void brgemm_1x1_convolution_fwd_t<isa>::execute_os_blocking(
 template <cpu_isa_t isa>
 void brgemm_1x1_convolution_fwd_t<isa>::execute_full_spatial(
         const brgemm_exec_ctx_t &brgemm_ctx,
-        brgemm_batch_element_t *const brg_batch_global, const float *dst_scales,
-        const float *oscales, int32_t src_zp_vals, int32_t *src_zp_comp,
-        int32_t *dst_zp_vals, int32_t *s8s8_compensation,
+        brgemm_batch_element_t *const brg_batch_global, const void *src_scales,
+        const void *wei_scales, const void *dst_scales, void *dst_scales_inv,
+        const int32_t *src_zero_points, int32_t *src_zp_comp,
+        const int32_t *dst_zero_points, int32_t *s8s8_compensation,
         char *const c_buffer_global) const {
 
     const auto &jcp = pd()->jcp_;
@@ -644,6 +680,15 @@ void brgemm_1x1_convolution_fwd_t<isa>::execute_full_spatial(
         char *const c_buffer = (jcp.use_buffer)
                 ? c_buffer_global + ithr * acc_dsz * jcp.LDC * jcp.M
                 : nullptr;
+
+        float *dst_scales_inv_ptr = nullptr;
+        if (jcp.with_dst_scales) {
+            const float *dst_scales_ptr
+                    = static_cast<const float *>(dst_scales);
+            dst_scales_inv_ptr = static_cast<float *>(dst_scales_inv) + ithr;
+            dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+        }
+
         int last_brg_idx = -1;
         int start {0}, end {0};
         balance211(work_amount, nthr, ithr, start, end);
@@ -662,9 +707,9 @@ void brgemm_1x1_convolution_fwd_t<isa>::execute_full_spatial(
             for (int icc = 0; icc < pd()->ic_chunks_; icc++) {
                 const int ow = owb * jcp.ow_block;
                 exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer, nullptr, g, n,
-                        ocb, od, oh, ow, icc, &last_brg_idx, oscales,
-                        src_zp_vals, src_zp_comp, dst_zp_vals,
-                        s8s8_compensation, dst_scales);
+                        ocb, od, oh, ow, icc, &last_brg_idx, src_zero_points,
+                        src_zp_comp, dst_zero_points, s8s8_compensation,
+                        src_scales, wei_scales, dst_scales_inv_ptr);
             }
             if (jcp.loop_order == loop_ndhwgc)
                 nd_iterator_step(n, jcp.mb, od, OD, oh, OH, owb, jcp.nb_ow, g,
@@ -690,16 +735,17 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
     const auto &jcp = pd()->jcp_;
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
 
-    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
-    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
-    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+    const void *src_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const void *wei_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const void *dst_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
 
-    const float *oscales = scale_utils::precompute_scales(scratchpad,
-            src_scales, wei_scales, pd()->OC(), pd()->attr(),
-            jit_scale_precompute_.get(), jcp.scale_adjust_factor);
-
-    DEFINE_ZERO_POINT_VALUE(src_zero_point, DNNL_ARG_SRC);
-    DEFINE_ZERO_POINT_VALUE(dst_zero_point, DNNL_ARG_DST);
+    const int32_t *src_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const int32_t *dst_zero_points = CTX_IN_MEM(
+            const int32_t *, DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
 
     const auto extra_data_offset
             = weights_d.size() - weights_d.additional_buffer_size();
@@ -713,7 +759,6 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
                                     ? jcp.s8s8_comp_buffer_size
                                     : 0)
             : nullptr;
-    int32_t *dst_zp_vals = jcp.dst_zero_point ? &dst_zero_point : nullptr;
 
     brgemm_batch_element_t *const brg_batch_global
             = (jcp.brg_type != brgemm_strd)
@@ -729,14 +774,19 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
     uint8_t *inp_buffer_mask_base = (jcp.is_rtus)
             ? scratchpad.template get<uint8_t>(key_conv_brgemm_inp_buffer_mask)
             : nullptr;
+    void *dst_scales_inv = jcp.with_dst_scales
+            ? scratchpad.template get<void>(key_conv_dst_scales)
+            : nullptr;
 
     if (jcp.is_os_blocking) {
-        execute_os_blocking(brgemm_ctx, brg_batch_global, dst_scales, oscales,
-                src_zero_point, zp_compensation, dst_zp_vals, s8s8_compensation,
+        execute_os_blocking(brgemm_ctx, brg_batch_global, src_scales,
+                wei_scales, dst_scales, dst_scales_inv, src_zero_points,
+                zp_compensation, dst_zero_points, s8s8_compensation,
                 c_buffer_global, inp_buffer_base, inp_buffer_mask_base);
     } else {
-        execute_full_spatial(brgemm_ctx, brg_batch_global, dst_scales, oscales,
-                src_zero_point, zp_compensation, dst_zp_vals, s8s8_compensation,
+        execute_full_spatial(brgemm_ctx, brg_batch_global, src_scales,
+                wei_scales, dst_scales, dst_scales_inv, src_zero_points,
+                zp_compensation, dst_zero_points, s8s8_compensation,
                 c_buffer_global);
     }
 
@@ -752,6 +802,8 @@ template struct brgemm_1x1_convolution_fwd_t<avx512_core_bf16>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core_fp16>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core_amx>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core_amx_fp16>;
+template struct brgemm_1x1_convolution_fwd_t<avx10_2_512>;
+template struct brgemm_1x1_convolution_fwd_t<avx10_2_512_amx_2>;
 
 } // namespace x64
 } // namespace cpu
