@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -90,13 +90,12 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
 
     const auto &jbgp = pd()->jbgp_;
 
-    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
-    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
-    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
-
-    const float *oscales
-            = scale_utils::precompute_scales(scratchpad, src_scales, wei_scales,
-                    pd()->OC(), pd()->attr(), jit_scale_precompute_.get());
+    const void *src_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const void *wei_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const void *dst_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
 
     const size_t src_dt_size = types::data_type_size(jbgp.src_dt);
     const size_t bia_dt_size
@@ -121,9 +120,9 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
     const int ic_chunks = div_up(jbgp.nb_ic, jbgp.nb_ic_blocking);
 
     const bool are_post_ops_applicable = one_of(true, jbgp.with_sum,
-            jbgp.with_bias, jbgp.with_scales, jbgp.with_eltwise,
-            jbgp.with_binary, jbgp.acc_dt != jbgp.dst_dt,
-            jbgp.req_s8s8_compensation, jbgp.with_dst_scales);
+            jbgp.with_bias, jbgp.with_src_scales, jbgp.with_wei_scales,
+            jbgp.with_dst_scales, jbgp.with_eltwise, jbgp.with_binary,
+            jbgp.acc_dt != jbgp.dst_dt, jbgp.req_s8s8_compensation);
     const bool can_use_dst_as_acc_buffer
             = jbgp.acc_dt == jbgp.dst_dt && !jbgp.with_sum;
     const int acc_buffer_idx_shift = can_use_dst_as_acc_buffer;
@@ -141,7 +140,7 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
     const auto ker = [&](int ithr_oc_mb, int nthr_oc_mb, int ithr_ic, int osb,
                              int osb_s, int ocb, int ocb_s, int icc, int icc_s,
                              int kd, int kh, int kw, bool copy_buffer_a,
-                             int &prev_ker_idx) {
+                             int &prev_ker_idx, const void *dst_scales_ptr) {
         const int cur_ocb = ocb + ocb_s;
         const int cur_osb = osb + osb_s;
         const int cur_icc = icc + icc_s;
@@ -281,10 +280,13 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
                         = jbgp.with_bias ? bias + bia_dt_size * oc : nullptr;
                 const brgemm_post_ops_data_t post_ops_data {
                         static_cast<const void *>(ptr_bias),
-                        &oscales[jbgp.is_oc_scale * oc],
                         post_ops_binary_rhs_arg_vec.data(),
                         static_cast<size_t>(oc), 0, dst, 0, nullptr, nullptr,
-                        nullptr, false, 1, false, false, dst_scales};
+                        nullptr, false, 1, false, false, src_scales,
+                        wei_scales ? static_cast<const char *>(wei_scales)
+                                        + jbgp.is_oc_scale * oc * sizeof(float)
+                                   : nullptr,
+                        dst_scales};
 
                 brgemm_kernel_execute_postops(brg_kernel, gemm_batch,
                         addr_batch, (void *)ptr_C, (void *)ptr_D, post_ops_data,
@@ -304,7 +306,8 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
             brgemm_palettes_.maybe_tile_configure(
                     is_amx, prev_ker_idx, brg_ker_ic_tail_idx);
 
-            int ic_block = gemm_batch * ic_blocks_per_batch;
+            const dim_t ic_block
+                    = static_cast<dim_t>(gemm_batch) * ic_blocks_per_batch;
             addr_batch[0].ptr.A = src
                     + blk_off(src_d, n, ic + ic_block * jbgp.ic_block, kd, kh,
                             kw);
@@ -326,10 +329,13 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
                         = jbgp.with_bias ? bias + bia_dt_size * oc : nullptr;
                 const brgemm_post_ops_data_t post_ops_data {
                         static_cast<const void *>(ptr_bias),
-                        &oscales[jbgp.is_oc_scale * oc],
                         post_ops_binary_rhs_arg_vec.data(),
                         static_cast<size_t>(oc), 0, dst, 0, nullptr, nullptr,
-                        nullptr, false, 1, false, false, dst_scales};
+                        nullptr, false, 1, false, false, src_scales,
+                        wei_scales ? static_cast<const char *>(wei_scales)
+                                        + jbgp.is_oc_scale * oc * sizeof(float)
+                                   : nullptr,
+                        dst_scales};
 
                 brgemm_kernel_execute_postops(brg_kernel_ic_tail, 1, addr_batch,
                         (void *)ptr_C, (void *)ptr_D, post_ops_data, scratch);
@@ -376,6 +382,16 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
         if (nthr_ic > 1)
             balance211(ic_chunks, nthr_ic, ithr_ic, icc_start, icc_end);
 
+        float *dst_scales_inv_ptr = nullptr;
+        if (jbgp.with_dst_scales) {
+            const float *dst_scales_ptr
+                    = static_cast<const float *>(dst_scales);
+            dst_scales_inv_ptr
+                    = scratchpad.template get<float>(key_conv_dst_scales)
+                    + ithr;
+            dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+        }
+
         const int icc_work = icc_end - icc_start;
 
         const auto order = jbgp.loop_order;
@@ -449,7 +465,7 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
                 for (int kw = 0; kw < jbgp.kw; kw++) {
                     ker(ithr_oc_mb, nthr_oc_mb, ithr_ic, osb, osb_s, ocb, ocb_s,
                             icc, icc_start, kd, kh, kw, copy_buffer_a,
-                            prev_ker_idx);
+                            prev_ker_idx, dst_scales_inv_ptr);
                 }
 
                 ++loop_start;
@@ -505,12 +521,6 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
     });
 
     if (jbgp.nthr_ic_b > 1) {
-        const bool is_f32
-                = everyone_is(f32, jbgp.src_dt, jbgp.wei_dt, jbgp.dst_dt);
-        const bool is_f32_compute = is_f32 && !jbgp.is_bf32;
-        MAYBE_UNUSED(is_f32_compute);
-        assert(jbgp.use_buffer && is_f32_compute);
-
         const auto get_dst_reduced_off = [&](int ithr_ic, int osb, int ocb) {
             assert(jbgp.nthr_ic_b > 1);
             int os = osb * jbgp.os_block;
@@ -607,11 +617,16 @@ status_t brgemm_inner_product_fwd_t<isa>::execute_forward(
 
                             const brgemm_post_ops_data_t post_ops_data {
                                     static_cast<const void *>(ptr_bias),
-                                    &oscales[jbgp.is_oc_scale * oc],
                                     post_ops_binary_rhs_arg_vec.data(),
                                     static_cast<size_t>(oc), 0, dst, 0, nullptr,
                                     nullptr, nullptr, true /* skip_accm */, 1,
-                                    false, false, dst_scales};
+                                    false, false, src_scales,
+                                    wei_scales ? static_cast<const char *>(
+                                                         wei_scales)
+                                                    + jbgp.is_oc_scale * oc
+                                                            * sizeof(float)
+                                               : nullptr,
+                                    dst_scales};
 
                             brgemm_kernel_execute_postops(brg_kernel, 0,
                                     nullptr, (void *)ptr_C, (void *)ptr_D,
@@ -637,6 +652,8 @@ template struct brgemm_inner_product_fwd_t<avx512_core_vnni>;
 template struct brgemm_inner_product_fwd_t<avx512_core_amx>;
 template struct brgemm_inner_product_fwd_t<avx512_core_fp16>;
 template struct brgemm_inner_product_fwd_t<avx512_core_amx_fp16>;
+template struct brgemm_inner_product_fwd_t<avx10_2_512>;
+template struct brgemm_inner_product_fwd_t<avx10_2_512_amx_2>;
 
 template <cpu_isa_t isa>
 void brgemm_inner_product_bwd_data_t<isa>::execute_backward_data(
@@ -1023,6 +1040,7 @@ template struct brgemm_inner_product_bwd_data_t<avx512_core_amx>;
 template struct brgemm_inner_product_bwd_data_t<avx512_core_bf16>;
 template struct brgemm_inner_product_bwd_data_t<avx512_core_amx_fp16>;
 template struct brgemm_inner_product_bwd_data_t<avx512_core_fp16>;
+template struct brgemm_inner_product_bwd_data_t<avx10_2_512>;
 
 template <cpu_isa_t isa>
 struct brgemm_inner_product_bwd_weights_t<isa>::thread_info_t {
@@ -1241,7 +1259,7 @@ void brgemm_inner_product_bwd_weights_t<isa>::transpose_matrix_c_chunk(
     const auto &jbgp = pd()->jbgp_;
 
     if (jbgp.is_amx) {
-        auto p = jit_amx_ip_trans_diff_wei::ctx_t();
+        auto p = jit_amx_ip_trans_diff_wei_t::ctx_t();
 
         // Note: This assumes AxB{inner_blocking} weights memory format.
         const dim_t ext_nb_ic = div_up(jbgp.ic, ext_ic_block_);
@@ -1791,6 +1809,7 @@ template struct brgemm_inner_product_bwd_weights_t<avx512_core_amx>;
 template struct brgemm_inner_product_bwd_weights_t<avx512_core_bf16>;
 template struct brgemm_inner_product_bwd_weights_t<avx512_core>;
 template struct brgemm_inner_product_bwd_weights_t<avx2>;
+template struct brgemm_inner_product_bwd_weights_t<avx10_2_512>;
 
 } // namespace x64
 } // namespace cpu
