@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2024 Intel Corporation
+* Copyright 2020-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -74,18 +74,29 @@ status_t jit_uni_resampling_fwd_t::pd_t::init(engine_t *engine) {
     fill_format_tag_info();
     conf_.isa = get_supported_isa(src_d.is_plain());
 
-    const bool ok = is_fwd() && !has_zero_dim_memory()
-            && conf_.src_tag != format_tag::undef
-            && set_default_params(conf_.src_tag) == status::success
-            && impl_supports_datatype(conf_.src_data_type)
-            && impl_supports_datatype(conf_.dst_data_type)
-            && IMPLICATION(conf_.src_data_type == f16, src_d.is_plain())
-            && attr()->has_default_values(sm::post_ops, conf_.dst_data_type)
-            && attr_.set_default_formats(dst_md(0)) == status::success;
-    if (!ok) return status::unimplemented;
-
-    if (!memory_desc_matches_tag(*dst_md(), conf_.src_tag))
-        return status::unimplemented;
+    VDISPATCH_RESAMPLING(is_fwd(), VERBOSE_BAD_PROPKIND);
+    VDISPATCH_RESAMPLING(!has_zero_dim_memory(), VERBOSE_EMPTY_TENSOR, "src");
+    VDISPATCH_RESAMPLING(
+            conf_.src_tag != format_tag::undef, VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_RESAMPLING(set_default_params(conf_.src_tag) == status::success,
+            VERBOSE_UNSUPPORTED_TAG);
+    VDISPATCH_RESAMPLING(impl_supports_datatype(conf_.src_data_type),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_RESAMPLING(impl_supports_datatype(conf_.dst_data_type),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_RESAMPLING(
+            IMPLICATION(conf_.src_data_type == f16, src_d.is_plain()),
+            VERBOSE_UNSUPPORTED_DT);
+    VDISPATCH_RESAMPLING(
+            attr()->has_default_values(sm::post_ops, conf_.dst_data_type),
+            VERBOSE_UNSUPPORTED_ATTR);
+    VDISPATCH_RESAMPLING(
+            attr_.set_default_formats(dst_md(0)) == status::success,
+            VERBOSE_UNSUPPORTED_POSTOP);
+    VDISPATCH_RESAMPLING(memory_desc_matches_tag(*dst_md(), conf_.src_tag),
+            VERBOSE_UNSUPPORTED_TAG_S, "dst");
+    VDISPATCH_RESAMPLING(impl::is_dense_format_kind({src_md(), dst_md()}),
+            VERBOSE_UNSUPPORTED_SPARSE_CFG);
 
     conf_.alg = desc()->alg_kind;
     conf_.c = C();
@@ -135,7 +146,8 @@ status_t jit_uni_resampling_fwd_t::pd_t::init(engine_t *engine) {
             attr()->post_ops_, &dst_d, sum_at_0_pos_only,
             sum_requires_scale_one, sum_requires_zp_zero,
             sum_requires_same_params, accepted_broadcasts);
-    if (!post_ops_ok(post_ops_args)) return status::unimplemented;
+    VDISPATCH_RESAMPLING(
+            post_ops_ok(post_ops_args), VERBOSE_UNSUPPORTED_POSTOP);
 
     conf_.post_ops = attr()->post_ops_;
 
@@ -154,6 +166,13 @@ status_t jit_uni_resampling_fwd_t::pd_t::init(engine_t *engine) {
     conf_.with_postops
             = conf_.with_eltwise || conf_.with_binary || conf_.with_sum;
 
+    VDISPATCH_RESAMPLING(
+            IMPLICATION(conf_.with_binary,
+                    !binary_injector::
+                            any_binary_postop_rhs_with_ternary_scalar_bcast(
+                                    conf_.post_ops, dst_d)),
+            VERBOSE_UNSUPPORTED_POSTOP);
+
     return status::success;
 }
 
@@ -446,7 +465,7 @@ status_t jit_uni_resampling_fwd_t::interpolate_nearest(const uint8_t *src,
             const dim_t dst_off = ((mb * C + c) * OD * OH * OW + od * OH * OW)
                     * dst_dt_size;
 
-            jit_resampling_call_s args = jit_resampling_call_s();
+            jit_uni_resampling_args_t args;
             args.src = src + src_off;
             args.dst = dst + dst_off;
             args.dst_orig = dst;
@@ -467,7 +486,7 @@ status_t jit_uni_resampling_fwd_t::interpolate_nearest(const uint8_t *src,
 
             const size_t cb = std::div(nsp, CB).rem;
 
-            jit_resampling_call_s args = jit_resampling_call_s();
+            jit_uni_resampling_args_t args;
             args.batch_of_sp_points_to_process = OW;
             args.src = src + src_off;
             args.dst = dst + dst_off;
@@ -508,7 +527,7 @@ status_t jit_uni_resampling_fwd_t::interpolate_linear(const uint8_t *src,
             const dim_t src_off = (mb * C + c) * ID * IH * IW * src_dt_size;
             const dim_t dst_off = (mb * C + c) * OD * OH * OW * dst_dt_size;
 
-            jit_resampling_call_s args = jit_resampling_call_s();
+            jit_uni_resampling_args_t args;
             args.batch_of_sp_points_to_process = OW * OH * OD;
             args.src = src + src_off;
             args.dst = dst + dst_off;
@@ -539,7 +558,7 @@ status_t jit_uni_resampling_fwd_t::interpolate_linear(const uint8_t *src,
 
             const size_t cb = std::div(nsp, CB).rem;
 
-            jit_resampling_call_s args = jit_resampling_call_s();
+            jit_uni_resampling_args_t args;
             args.batch_of_sp_points_to_process = OW;
             args.src = src + src_off;
             args.dst = dst + dst_off;
