@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2023 Intel Corporation
+* Copyright 2019-2025 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -51,22 +51,22 @@ using namespace data_type;
 
 template <cpu_isa_t isa>
 struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
-                                    public jit_generator {
+                                    public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_softmax_dense_kernel_t)
 
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const AddressFrame &vmmword = is_superset(isa, avx512_core) ? zword
             : is_superset(isa, avx)                             ? yword
                                                                 : xword;
-    static constexpr auto vlen = cpu_isa_traits<isa>::vlen;
-    static constexpr auto n_vregs = cpu_isa_traits<isa>::n_vregs;
+    static constexpr auto vlen = cpu_isa_traits_t<isa>::vlen;
+    static constexpr auto n_vregs = cpu_isa_traits_t<isa>::n_vregs;
     static constexpr auto simd_w_ = vlen / sizeof(float); // bf16 works on ymms
 
     const memory_desc_wrapper src_d_, dst_d_, diff_dst_d_;
     io::jit_io_multi_dt_helper_t<Vmm> io_;
 
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> exp_injector_;
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> log_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_t<isa>> exp_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_t<isa>> log_injector_;
     std::unique_ptr<injector::jit_uni_postops_injector_t<isa>>
             postops_injector_;
 
@@ -84,6 +84,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
     Reg64 reg_tmp = r13;
     Reg64 reg_dst_spat_offt = r15;
     Reg64 reg_diff_dst_spat_offt = reg_log_injector_table;
+    Reg64 reg_tmp2 = reg_log_injector_table;
     Reg64 reg_interim = reg_diff_dst;
     Reg64 reg_interim_spat_offt = abi_not_param1;
     Reg64 reg_src_scales = rsi;
@@ -136,13 +137,22 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
     const int bf16_emu_zmm_4_idx_ = 26;
     const int tail_opmask_idx_ = 2;
 
+    const int fp8_emu_zmm_1_idx_ = 23;
+    const int fp8_emu_zmm_2_idx_ = 24;
+    const int fp8_emu_zmm_3_idx_ = 25;
+    const int fp8_emu_zmm_4_idx_ = 26;
+    const int fp8_emu_zmm_5_idx_ = 27;
+    const int fp8_emu_kmask_idx_ = 3;
+
     Opmask tail_opmask = Opmask(tail_opmask_idx_);
 
     void operator()(const call_params_t *p) const override {
-        return jit_generator::operator()(p);
+        return jit_generator_t::operator()(p);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     bool is_data_type_xf16(data_type_t dt) {
         return utils::one_of(dt, bf16, f16);
@@ -562,8 +572,24 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
         axis_loop(pre_body, body, post_body);
 
         get_horizontal_op(vsum, vtmp = vmax, op_t::sum);
-        if (is_softmax_) uni_vdivps(vsum, vone, vsum, vtmp = vmax);
-        if (is_logsoftmax_) log_injector_->compute_vector(vsum.getIdx());
+
+        if (pd_->alg_kind() == alg_kind::softmax_accurate_inf_as_zero) {
+            Xbyak::Label skip_div;
+            // `vptest` sets the `ZF` flag if all bits in the result are 0 of
+            // the bitwise AND of source operands.
+            // Note: using Vmm(1) is an ugly workaround EVEX versus VEX encoding
+            // as `vsum` uses index `30` on avx512_core. `Vmm(1)` is a tmp vreg
+            // used to read data just above. Should be safe.
+            uni_vmovups(Vmm(1), vsum);
+            uni_vptest(Xmm(1), Xmm(1));
+            jz(skip_div, T_NEAR); // Check if ZF is set.
+            uni_vdivps(vsum, vone, vsum, vtmp = vmax);
+            L(skip_div);
+        } else if (is_softmax_) {
+            uni_vdivps(vsum, vone, vsum, vtmp = vmax);
+        } else if (is_logsoftmax_) {
+            log_injector_->compute_vector(vsum.getIdx());
+        }
     }
 
     void accumulate_vsum() {
@@ -609,7 +635,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                     // Prepare indices for exp aux vmms.
                     injector_utils::vmm_index_set_t exp_aux_indices;
                     const auto exp_vmm_aux_count
-                            = jit_uni_eltwise_injector_f32<isa>::aux_vecs_count(
+                            = jit_uni_eltwise_injector_t<isa>::aux_vecs_count(
                                     alg_kind::eltwise_exp, pd_->is_fwd(), 0.f);
                     for (size_t j = 0; j < exp_vmm_aux_count; j++) {
                         // Insert the next idx starting after `vreg_tmp_sum`.
@@ -673,8 +699,23 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
 
         get_horizontal_op(vsum, vtmp = vmax, op_t::sum);
 
-        if (is_softmax_) uni_vdivps(vsum, vone, vsum, vtmp = vmax);
-        if (is_logsoftmax_) log_injector_->compute_vector(vsum.getIdx());
+        if (pd_->alg_kind() == alg_kind::softmax_accurate_inf_as_zero) {
+            Xbyak::Label skip_div;
+            // `vptest` sets the `ZF` flag if all bits in the result are 0 of
+            // the bitwise AND of source operands.
+            // Note: using Vmm(1) is an ugly workaround EVEX versus VEX encoding
+            // as `vsum` uses index `30` on avx512_core. `Vmm(1)` is a tmp vreg
+            // used to read data just above. Should be safe.
+            uni_vmovups(Vmm(1), vsum);
+            uni_vptest(Xmm(1), Xmm(1));
+            jz(skip_div, T_NEAR); // Check if ZF is set.
+            uni_vdivps(vsum, vone, vsum, vtmp = vmax);
+            L(skip_div);
+        } else if (is_softmax_) {
+            uni_vdivps(vsum, vone, vsum, vtmp = vmax);
+        } else if (is_logsoftmax_) {
+            log_injector_->compute_vector(vsum.getIdx());
+        }
     }
 
     // Use ne_convert instruction to load xf16 even/odd elements from memory
@@ -687,21 +728,31 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                 Vmm vreg_tmp_src_even = Vmm(i + 1);
                 Vmm vreg_tmp_src_odd = Vmm(i + 2);
                 vtmp = Vmm(i + 3);
-                if (can_load_two_simdw && !need_scratchpad_) {
-                    io_[dst_d_.data_type()]->load_two_simdw_xf16(
-                            dst_ptr(dst_next_vreg_stride_ * i),
-                            vreg_tmp_src_even, vreg_tmp_src_odd);
-                    io_[dst_d_.data_type()]->merge_interleaved_to_plain(
-                            vreg_tmp_src_even, vreg_tmp_src_odd, vtmp);
-                } else {
-                    if (need_scratchpad_)
+                if (can_load_two_simdw) {
+                    if (!need_scratchpad_) {
+                        io_[dst_d_.data_type()]->load_two_simdw_xf16(
+                                dst_ptr(dst_next_vreg_stride_ * i),
+                                vreg_tmp_src_even, vreg_tmp_src_odd);
+                        io_[dst_d_.data_type()]->merge_interleaved_to_plain(
+                                vreg_tmp_src_even, vreg_tmp_src_odd, vtmp);
+                    } else {
                         io_[f32]->load(
                                 interim_ptr(interim_next_vreg_stride_ * i),
                                 vreg_tmp_src_even, tail);
-                    else
+                        io_[f32]->load(interim_ptr(interim_next_vreg_stride_
+                                               * (i + 1)),
+                                vreg_tmp_src_odd, tail);
+                    }
+                } else {
+                    if (!need_scratchpad_) {
                         io_[dst_d_.data_type()]->load(
                                 dst_ptr(dst_next_vreg_stride_ * i),
                                 vreg_tmp_src_even, tail);
+                    } else {
+                        io_[f32]->load(
+                                interim_ptr(interim_next_vreg_stride_ * i),
+                                vreg_tmp_src_even, tail);
+                    }
                 }
                 for (int i_odd = 0; i_odd < 2 && i_odd + i < unroll; i_odd++) {
                     const auto vreg_tmp_src
@@ -713,7 +764,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
 
                     if (with_src_scales_) {
                         Vmm vscale = vmax;
-                        uni_vmovups(vscale, ptr[reg_src_scales]);
+                        uni_vbroadcastss(vscale, ptr[reg_src_scales]);
                         uni_vmulps(vreg_tmp_src, vreg_tmp_src, vscale);
                     }
                     if (with_postops_) {
@@ -734,7 +785,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                     }
                     if (with_dst_scales_) {
                         Vmm vscale = vmax;
-                        uni_vmovups(vscale, ptr[reg_dst_scales]);
+                        uni_vbroadcastss(vscale, ptr[reg_dst_scales]);
                         uni_vmulps(vreg_tmp_src, vreg_tmp_src, vscale);
                     }
 
@@ -777,7 +828,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                     uni_vsubps(vreg_tmp_src, vreg_tmp_src, vsum);
 
                 if (with_src_scales_) {
-                    uni_vmovups(vreg_tmp_scale, ptr[reg_src_scales]);
+                    uni_vbroadcastss(vreg_tmp_scale, ptr[reg_src_scales]);
                     uni_vmulps(vreg_tmp_src, vreg_tmp_src, vreg_tmp_scale);
                 }
                 if (with_postops_) {
@@ -796,7 +847,7 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                             vreg_tmp_src.getIdx(), rhs_arg_params);
                 }
                 if (with_dst_scales_) {
-                    uni_vmovups(vreg_tmp_scale, ptr[reg_dst_scales]);
+                    uni_vbroadcastss(vreg_tmp_scale, ptr[reg_dst_scales]);
                     uni_vmulps(vreg_tmp_src, vreg_tmp_src, vreg_tmp_scale);
                 }
             }
@@ -889,18 +940,18 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
     // initialization.
     void generate() override {
         if (pd_->is_fwd() || is_logsoftmax_)
-            exp_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, !use_ext_aux_vmms_,
-                    reg_exp_injector_table, injector_mask));
+            exp_injector_.reset(new jit_uni_eltwise_injector_t<isa>(this,
+                    alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, data_type::f32,
+                    !use_ext_aux_vmms_, reg_exp_injector_table, injector_mask));
         if (pd_->is_fwd() && is_logsoftmax_) {
-            log_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, true,
-                    reg_log_injector_table, injector_mask));
+            log_injector_.reset(new jit_uni_eltwise_injector_t<isa>(this,
+                    alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, data_type::f32,
+                    true, reg_log_injector_table, injector_mask));
         }
         if (with_postops_) {
             static constexpr bool preserve_gpr = true;
             static constexpr bool preserve_vmm = true;
-            static constexpr bool use_exact_tail_scalar_bcast = true;
+            static constexpr bool use_exact_tail_scalar_bcast = false;
             static constexpr std::size_t tmp_vmm_injector = 0u;
 
             const binary_injector::rhs_arg_static_params_t rhs_sp {
@@ -934,12 +985,13 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
         if (exp_injector_) exp_injector_->prepare_table();
         if (log_injector_) log_injector_->prepare_table();
         if (with_eltwise_ && postops_injector_)
-            postops_injector_->prepare_table();
+            postops_injector_->prepare_table(/* generate = */ true);
+        io_.prepare_table_fp8();
     }
 
     jit_softmax_dense_kernel_t(const softmax_pd_t *pd)
         : jit_softmax_kernel_base_t(pd)
-        , jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
+        , jit_generator_t(jit_name(), isa)
         , src_d_(pd_->invariant_src_md())
         , dst_d_(pd_->dst_md())
         , diff_dst_d_(pd_->diff_dst_md())
@@ -947,7 +999,14 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
         , is_f16_(utils::one_of(f16, src_d_.data_type(), dst_d_.data_type()))
         , is_avx2_ne_xf16_(mayiuse(avx2_vnni_2) && !mayiuse(avx512_core)
                   && (is_bf16_ || is_f16_))
-        , need_scratchpad_(utils::one_of(dst_d_.data_type(), u8, s8))
+        // Note: must be aligned with pd_t::init()->init_scratchpad();
+        , need_scratchpad_(pd_->is_fwd() && dst_d_.data_type() != f32
+                  && /* !relaxed_acc */ !(
+                          src_d_.data_type() == dst_d_.data_type()
+                          && !types::is_integral_dt(dst_d_.data_type())
+                          && utils::one_of(pd_->attr()->acc_mode_,
+                                  accumulation_mode::relaxed,
+                                  accumulation_mode::any)))
         , use_ext_aux_vmms_(!is_logsoftmax_ && n_vregs > 16)
         , axis_simd_full_(pd_->axis_size() / simd_w_)
         , axis_simd_tail_(pd_->axis_size() % simd_w_) {
@@ -959,9 +1018,9 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
 
         const auto &attr_scales = pd_->attr()->scales_;
         with_src_scales_ = is_superset(isa, avx2)
-                && !attr_scales.get(DNNL_ARG_SRC).has_default_values();
+                && !attr_scales.has_default_values(DNNL_ARG_SRC);
         with_dst_scales_ = is_superset(isa, avx2)
-                && !attr_scales.get(DNNL_ARG_DST).has_default_values();
+                && !attr_scales.has_default_values(DNNL_ARG_DST);
 
         io::io_conf_t io_conf;
         io::io_tail_conf_t io_tail_conf(simd_w_, axis_simd_tail_,
@@ -971,30 +1030,34 @@ struct jit_softmax_dense_kernel_t : jit_softmax_kernel_base_t,
                 bf16_emu_zmm_4_idx_);
         io::io_saturation_conf_t io_saturation_conf(
                 vzero.getIdx(), vsaturation_ubound.getIdx(), reg_tmp);
+        io::io_emu_fp8_conf_t io_fp8_conf(fp8_emu_zmm_1_idx_,
+                fp8_emu_zmm_2_idx_, fp8_emu_zmm_3_idx_, fp8_emu_zmm_4_idx_,
+                fp8_emu_zmm_5_idx_, fp8_emu_kmask_idx_, reg_tmp);
         io_ = io::jit_io_multi_dt_helper_t<Vmm>(this, isa,
                 {src_d_.data_type(), dst_d_.data_type(), f32 /* stats */},
                 io_conf, io_tail_conf, io_bf16_conf,
-                {{dst_d_.data_type(), io_saturation_conf}});
+                {{dst_d_.data_type(), io_saturation_conf}}, utils::nullopt,
+                io_fp8_conf);
     }
 };
 
 template <cpu_isa_t isa>
 struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
-                                      public jit_generator {
+                                      public jit_generator_t {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_softmax_strided_kernel_t)
 
-    using Vmm = typename cpu_isa_traits<isa>::Vmm;
+    using Vmm = typename cpu_isa_traits_t<isa>::Vmm;
     const AddressFrame &vmmword = is_superset(isa, avx512_core) ? zword
             : is_superset(isa, avx)                             ? yword
                                                                 : xword;
-    static constexpr auto vlen = cpu_isa_traits<isa>::vlen;
+    static constexpr auto vlen = cpu_isa_traits_t<isa>::vlen;
     static constexpr auto simd_w_ = vlen / sizeof(float); // bf16 works on ymms
 
     const memory_desc_wrapper src_d_, dst_d_;
     io::jit_io_multi_dt_helper_t<Vmm> io_;
 
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> exp_injector_;
-    std::unique_ptr<jit_uni_eltwise_injector_f32<isa>> log_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_t<isa>> exp_injector_;
+    std::unique_ptr<jit_uni_eltwise_injector_t<isa>> log_injector_;
     std::unique_ptr<injector::jit_uni_postops_injector_t<isa>>
             postops_injector_;
 
@@ -1010,6 +1073,7 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
     Reg64 reg_interim_spat_offt = rsi;
     Reg64 reg_reverse_n_elems = r12;
     Reg64 reg_tmp = r13;
+    Reg64 reg_tmp2 = reg_log_injector_table;
     Reg64 reg_interim = r14;
     Reg64 reg_reverse_axis_elems = r11;
 
@@ -1056,13 +1120,22 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
     const int bf16_emu_zmm_4_idx_ = 26;
     const int tail_opmask_idx_ = 2;
 
+    const int fp8_emu_zmm_1_idx_ = 23;
+    const int fp8_emu_zmm_2_idx_ = 24;
+    const int fp8_emu_zmm_3_idx_ = 25;
+    const int fp8_emu_zmm_4_idx_ = 26;
+    const int fp8_emu_zmm_5_idx_ = 27;
+    const int fp8_emu_kmask_idx_ = 3;
+
     Opmask tail_opmask = Opmask(tail_opmask_idx_);
 
     void operator()(const call_params_t *p) const override {
-        return jit_generator::operator()(p);
+        return jit_generator_t::operator()(p);
     }
 
-    status_t create_kernel() override { return jit_generator::create_kernel(); }
+    status_t create_kernel() override {
+        return jit_generator_t::create_kernel();
+    }
 
     void compute_predefined_variables() {
         // `axis_simd_full_` is actually `inner_simd_full_`.
@@ -1096,11 +1169,11 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
         }
         if (with_src_scales_) {
             mov(reg_tmp, ptr[reg_param + PARAM_OFF(src_scales)]);
-            uni_vmovups(vsrc_scale, ptr[reg_tmp]);
+            uni_vbroadcastss(vsrc_scale, ptr[reg_tmp]);
         }
         if (with_dst_scales_) {
             mov(reg_tmp, ptr[reg_param + PARAM_OFF(dst_scales)]);
-            uni_vmovups(vdst_scale, ptr[reg_tmp]);
+            uni_vbroadcastss(vdst_scale, ptr[reg_tmp]);
         }
     }
 
@@ -1346,8 +1419,24 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
             Vmm vreg_tmp_src = Vmm(i + 1);
             Vmm vtmp = get_vmax(vreg_tmp_src, unroll_inner);
             Vmm vsum = get_vsum(vreg_tmp_src, unroll_inner);
-            if (is_softmax_) uni_vdivps(vsum, vone, vsum, vtmp);
-            if (is_logsoftmax_) log_injector_->compute_vector(vsum.getIdx());
+
+            if (pd_->alg_kind() == alg_kind::softmax_accurate_inf_as_zero) {
+                Xbyak::Label skip_div;
+                // `vptest` sets the `ZF` flag if all bits in the result are 0
+                // of the bitwise AND of source operands.
+                // Note: using Vmm(1) is an ugly workaround EVEX versus VEX
+                // encoding as `vsum` uses index `30` on avx512_core. `Vmm(1)`
+                // is a tmp vreg used to read data just above. Should be safe.
+                uni_vmovups(Vmm(1), vsum);
+                uni_vptest(Xmm(1), Xmm(1));
+                jz(skip_div, T_NEAR); // Check if ZF is set.
+                uni_vdivps(vsum, vone, vsum, vtmp);
+                L(skip_div);
+            } else if (is_softmax_) {
+                uni_vdivps(vsum, vone, vsum, vtmp);
+            } else if (is_logsoftmax_) {
+                log_injector_->compute_vector(vsum.getIdx());
+            }
         }
 
         axis_size_loop_unroll(store_body, unroll_inner, tail);
@@ -1436,18 +1525,18 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
 
     void generate() override {
         if (pd_->is_fwd() || is_logsoftmax_)
-            exp_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, true,
-                    reg_exp_injector_table, injector_mask));
+            exp_injector_.reset(new jit_uni_eltwise_injector_t<isa>(this,
+                    alg_kind::eltwise_exp, 0.0f, 0.0f, 1.0f, data_type::f32,
+                    true, reg_exp_injector_table, injector_mask));
         if (pd_->is_fwd() && is_logsoftmax_) {
-            log_injector_.reset(new jit_uni_eltwise_injector_f32<isa>(this,
-                    alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, true,
-                    reg_log_injector_table, injector_mask));
+            log_injector_.reset(new jit_uni_eltwise_injector_t<isa>(this,
+                    alg_kind::eltwise_log, 0.0f, 0.0f, 1.0f, data_type::f32,
+                    true, reg_log_injector_table, injector_mask));
         }
         if (with_postops_) {
             static constexpr bool preserve_gpr = true;
             static constexpr bool preserve_vmm = true;
-            static constexpr bool use_exact_tail_scalar_bcast = true;
+            static constexpr bool use_exact_tail_scalar_bcast = false;
             static constexpr std::size_t tmp_vmm_injector = 0u;
 
             const binary_injector::rhs_arg_static_params_t rhs_sp {
@@ -1482,15 +1571,23 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
         if (exp_injector_) exp_injector_->prepare_table();
         if (log_injector_) log_injector_->prepare_table();
         if (with_eltwise_ && postops_injector_)
-            postops_injector_->prepare_table();
+            postops_injector_->prepare_table(/* generate = */ true);
+        io_.prepare_table_fp8();
     }
 
     jit_softmax_strided_kernel_t(const softmax_pd_t *pd)
         : jit_softmax_kernel_base_t(pd)
-        , jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
+        , jit_generator_t(jit_name(), isa)
         , src_d_(pd_->invariant_src_md())
         , dst_d_(pd_->dst_md())
-        , need_scratchpad_(utils::one_of(dst_d_.data_type(), u8, s8))
+        // Note: must be aligned with pd_t::init()->init_scratchpad();
+        , need_scratchpad_(pd_->is_fwd() && dst_d_.data_type() != f32
+                  && /* !relaxed_acc */ !(
+                          src_d_.data_type() == dst_d_.data_type()
+                          && !types::is_integral_dt(dst_d_.data_type())
+                          && utils::one_of(pd_->attr()->acc_mode_,
+                                  accumulation_mode::relaxed,
+                                  accumulation_mode::any)))
         , axis_size_(pd_->axis_size())
         // `axis_stride_`, `axis_simd_full_` and `axis_simd_tail_` are only
         // different pieces from the dense version.
@@ -1512,9 +1609,9 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
 
         const auto &attr_scales = pd_->attr()->scales_;
         with_src_scales_ = is_superset(isa, avx2)
-                && !attr_scales.get(DNNL_ARG_SRC).has_default_values();
+                && !attr_scales.has_default_values(DNNL_ARG_SRC);
         with_dst_scales_ = is_superset(isa, avx2)
-                && !attr_scales.get(DNNL_ARG_DST).has_default_values();
+                && !attr_scales.has_default_values(DNNL_ARG_DST);
 
         io::io_conf_t io_conf;
         io::io_tail_conf_t io_tail_conf(simd_w_, axis_simd_tail_,
@@ -1524,10 +1621,14 @@ struct jit_softmax_strided_kernel_t : jit_softmax_kernel_base_t,
                 bf16_emu_zmm_4_idx_);
         io::io_saturation_conf_t io_saturation_conf(
                 vzero.getIdx(), vsaturation_ubound.getIdx(), reg_tmp);
+        io::io_emu_fp8_conf_t io_fp8_conf(fp8_emu_zmm_1_idx_,
+                fp8_emu_zmm_2_idx_, fp8_emu_zmm_3_idx_, fp8_emu_zmm_4_idx_,
+                fp8_emu_zmm_5_idx_, fp8_emu_kmask_idx_, reg_tmp);
         io_ = io::jit_io_multi_dt_helper_t<Vmm>(this, isa,
                 {src_d_.data_type(), dst_d_.data_type(), f32 /* stats */},
                 io_conf, io_tail_conf, io_bf16_conf,
-                {{dst_d_.data_type(), io_saturation_conf}});
+                {{dst_d_.data_type(), io_saturation_conf}}, utils::nullopt,
+                io_fp8_conf);
     }
 };
 
@@ -1588,11 +1689,15 @@ status_t jit_uni_softmax_fwd_t::init(engine_t *engine) {
 status_t jit_uni_softmax_fwd_t::execute(const exec_ctx_t &ctx) const {
     const auto src = CTX_IN_MEM(const char *, DNNL_ARG_SRC);
     auto dst = CTX_OUT_MEM(char *, DNNL_ARG_DST);
-    auto scratchpad_ptr = ctx.get_scratchpad_grantor().template get<char>(
-            memory_tracking::names::key_softmax_interim_store);
 
-    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
-    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+    const void *src_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const void *dst_scales
+            = CTX_IN_MEM(const void *, DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+
+    const auto &scratchpad = ctx.get_scratchpad_grantor();
+    auto scratchpad_ptr = scratchpad.template get<char>(
+            memory_tracking::names::key_softmax_interim_store);
 
     const auto post_ops_binary_rhs_arg_vec
             = binary_injector::prepare_binary_args(
@@ -1661,6 +1766,17 @@ status_t jit_uni_softmax_fwd_t::execute(const exec_ctx_t &ctx) const {
                 char *interim_ptr = scratchpad_ptr
                         ? scratchpad_ptr + ithr * pd()->scratch_size_per_thr_
                         : nullptr;
+                float *dst_scales_inv_ptr = nullptr;
+                if (!pd()->attr()->scales_.has_default_values(DNNL_ARG_DST)) {
+                    const float *dst_scales_ptr
+                            = static_cast<const float *>(dst_scales);
+                    dst_scales_inv_ptr
+                            = scratchpad.template get<float>(memory_tracking::
+                                              names::key_softmax_dst_scales)
+                            + ithr;
+                    dst_scales_inv_ptr[0] = 1.f / dst_scales_ptr[0];
+                }
+
                 softmax_impl::jit_softmax_kernel_base_t::call_params_t p;
                 if (pd()->axis_is_plain_and_strided_ && outer_size == 1) {
                     // Special case when inner size is split between threads.
@@ -1676,7 +1792,7 @@ status_t jit_uni_softmax_fwd_t::execute(const exec_ctx_t &ctx) const {
                 p.dst = dst_ptr;
                 p.interim = interim_ptr;
                 p.src_scales = src_scales;
-                p.dst_scales = dst_scales;
+                p.dst_scales = dst_scales_inv_ptr;
                 // post-ops
                 p.dst_orig = dst_orig_ptr;
                 p.post_ops_binary_rhs_arg_vec
